#ifndef __ENVIRONMENT_H__INCLUDED__
#define __ENVIRONMENT_H__INCLUDED__

#include <vector>

#include <graphics/Sounds.h>
#include <graphics/Vector3.h>


using namespace graphics;


class Environment
{
public:
	typedef	Vector3<float>	Vector;


protected:
	class Location
	{
	public:
		Vector	vPosition;
		float	fMaxRadius;

		bool	bStarted,
				bPlaying;
		Sound	sndBackground;
		float	fMaxVolume,
				fVolume;

		// Nebelreichweite, -dichte und -farbe.
		float	fFogNear,
				fFogFar,
				fFogDensity,
				fFogR,
				fFogG,
				fFogB,
				fAmbR,
				fAmbG,
				fAmbB;

		// Lichteinfallswinkel (der Sonnenstrahlen)
		Vector	vLightSource;
		float	fTurbolence;


		Location(const Vector &position, const float &_fMaxRadius, const string &sample, const float &fMaxVolume = 128.0f);


		inline void	start()
		{	bPlaying = true;	}
		inline void	stop()
		{	bPlaying = false;	}


		void	advance(float time);
	};


	Scene				*pScene;

	vector<Location *>	vLocations;
	int_t				nCurrent;


	// Lichteinfallswinkel (der Sonnenstrahlen)
	Vector	vLightSource;
	float	fTurbolence;

	// Nebelreichweite, -dichte und -farbe.
	float	fFogNear,
			fFogFar,
			fFogDensity,
			fFogR,
			fFogG,
			fFogB,
			fAmbR,
			fAmbG,
			fAmbB;

	// sound ausschalten
	bool	bSilent;


public:
	 Environment();
	~Environment();


	void addEnvironment(const Vector &position, const float &fMaxRadius, const string &sample, const float &fMaxVolume = 128.0f);

	void addEnvironment(const Vector &position, const float &fMaxRadius, const string &sample, const float &fMaxVolume, const float &FogR, const float &FogG, const float &FogB, const float &FogDensity, const float &FogNear, const float &FogFar);

	void addEnvironment(const Vector &position, const float &fMaxRadius, const string &sample, const float &fMaxVolume, const float &FogR, const float &FogG, const float &FogB, const float &FogDensity, const float &FogNear, const float &FogFar, const float &AmbR, const float &AmbG, const float &AmbB);



	inline void	scene(Scene *_pScene)
	{	pScene = _pScene;	}

	void		silent(const bool _silent = true);


	void		advance(const Vector &position, float time);
};

#endif
