#ifndef __WEAPON_H__INCLUDED__
#define __WEAPON_H__INCLUDED__

#include "Projectile.h"


using namespace graphics;


// Das ist unsere Weapon-Klasse.
class Weapon
{
public:
	enum Type
	{
		LightMG,
		HeavyMG,
		LightCannon,
		HeavyCannon,
		GaussCannon,
		TorpedoLauncher,

		COUNT
	};


protected:
	static Sound		sndEmpty,
						sndLightMG,
						sndHeavyMG,
						sndLightCannon,
						sndHeavyCannon,
						sndGaussCannon,
						sndTorpedoLauncher;
	static const char	*strNames[COUNT];

	Vehicle		*parent;
	Type		type;
	uint_t		nAmmunition,
				nMaxAmmunition;
	float		fCooldown,
				fCooldownTimer,
				fProjectileLife,
				fProjectileSpeed,
				fProjectileDamage;


public:
	Weapon(Vehicle *_parent, const Type &_type);


	Projectile *	fire(const Vector3<float> &position, const Vector3<float> &forward, const float &speed);


	inline bool		ammoFull() const
	{	return nAmmunition >= nMaxAmmunition;	}
	inline bool		ready() const
	{	return (fCooldownTimer <= 0.0f) && (nAmmunition > 0);	}
	
	inline void		ammunition(const uint_t &ammo)
	{	nAmmunition = ammo;	}
	inline void		addAmmunition(const int_t &ammo)
	{	nAmmunition += ammo;
		if(nAmmunition > nMaxAmmunition)
			nAmmunition = nMaxAmmunition;	}
	inline void		refill()
	{	nAmmunition = nMaxAmmunition;	}

	inline uint_t		getAmmunition() const
	{	return nAmmunition;	}

	inline const char *	getName() const
	{	return strNames[type];	}


	inline void		advance(const float &time)
	{
		fCooldownTimer -= time;
		if(fCooldownTimer < 0.0f)
			fCooldownTimer = 0.0f;
	}
};


#endif
