#ifndef __FRAMEWORK__ALLOCATOR_H__INCLUDED__
#define __FRAMEWORK__ALLOCATOR_H__INCLUDED__

#include "_Allocator.h"


FRAMEWORK_BEGIN


/*
 * The _AllocatorBase creates and destroys Objects using the new and
 * delete operators. Additionally it provides functions for creating
 * objects using an "Id" - this could be a class containing all
 * parameters to be passed to the object's constructor -, copying
 * them and destroying them.
 */
template <class Item, class Id = void> class _AllocatorBase :
	public __AllocatorBase
{
public:
	typedef __AllocatorBase				_Base;
	typedef _AllocatorBase<Item, Id>	_Type;


public:
	inline _AllocatorBase<Item, Id>()
	{	/* nothing to do */	}

	Item *	_new() const
	{
		Item *pItem = new Item;
		if(!pItem)
			throw OutOfMemoryError();
		return pItem;
	}
	Item *	_new(const Id *pIdRef) const
	{
		Item *pItem = new Item(*pIdRef);
		if(!pItem)
			throw OutOfMemoryError();
		return pItem;
	}
	Item *	_copy(const Item *_pItem) const
	{
		Item *pItem = new Item(*_pItem);
		if(!pItem)
			throw OutOfMemoryError();
		return pItem;
	}
	inline void	_delete(Item *pItem) const
	{	delete pItem;	}
};


/*
 * The Allocator class acts like the allocator base, and it can be
 * created using __new(), copied with __copy() and destroyed using
 * __delete().
 */
template <class Item, class Id = void> class Allocator :
	public _AllocatorBase<Item, Id>
{
public:
	typedef _AllocatorBase<Item, Id>	_Base;
	typedef Allocator<Item, Id>			_Type;


	inline Allocator<Item, Id>()
	{	/* nothing to do */	}


	static inline _Type *	__new()
	{	return new _Type();	}
	_Type *					__copy() const
	{
		_Type *	pNew = new _Type;
		if(!pNew)
			throw OutOfMemoryError();
		return pNew;
	}
	inline void				__delete()
	{	delete this;	}
};


/*
 * The StaticAllocator class acts like the allocator base, but it can NOT be
 * created using __new() or __copy() and destroyed using __delete() like the
 * Allocator.
 */
template <class Item, class Id = void> class StaticAllocator :
	public _AllocatorBase<Item, Id>,
	public _StaticAllocatorBase<StaticAllocator<Item, Id> >
{
public:
	typedef _AllocatorBase<Item, Id>							_Base1;
	typedef _StaticAllocatorBase<StaticAllocator<Item, Id> >	_Base2;
	typedef StaticAllocator<Item, Id>							_Type;

	inline StaticAllocator<Item, Id>()
	{	/* nothing to do */	}
};


FRAMEWORK_END
#endif
