#ifndef __FRAMEWORK__ERROR_H__INCLUDED__
#define __FRAMEWORK__ERROR_H__INCLUDED__

#include <stdexcept>
#include <string>

#include "types.h"

#define getErrorMessage(message)\
	(std::string("in function ") + __FUNCTION__ + ": " + (message))

#ifndef _WINDOWS
#define getLastErrorString()\
	strerror()
#endif
#define	getLastErrorMessage(message)\
	(std::string("in function ") + __FUNCTION__ + ": " + (message) + ":\n\t" + getLastErrorString())

#define DEFINE_ERROR(error_class, text)\
	class error_class : public Error\
	{public:\
		inline error_class() : Error(text) {}\
		inline error_class(const string &str) :	Error(string(text) + ": " + str) {}	}


FRAMEWORK_BEGIN


class Error : public exception
{
public:
	inline Error(const char *str) :	exception(str)
	{	/* nothing to do */	}
	inline Error(const string &str) :	exception(str.c_str())
	{	/* nothing to do */	}

	Error &	operator =(const Error &e);

	void				report() const;

	inline const char *	toString() const
	{	return what();	}
};


DEFINE_ERROR(OutOfMemoryError, "Out Of Memory");
DEFINE_ERROR(NotImplementedError, "Not Implemented");
DEFINE_ERROR(NotSupportedError, "Not Supported");


FRAMEWORK_END
#endif
