#ifndef __GRAPHICS__CAMERA_H__INCLUDED__
#define __GRAPHICS__CAMERA_H__INCLUDED__

#include "TransformationMatrix3.h"
#include "Object3.h"

#include "SDLMutex.h"


GRAPHICS_BEGIN


class Camera
{
public:
	typedef Vector3<GLdouble>				Vector;
	typedef Vector3<GLfloat>				VectorF;
	typedef Vector4<GLfloat>				Vector4F;
	typedef TransformationMatrix3<GLdouble>	TransformationMatrix;

	enum Visibility
	{
		full,
		partial,
		none
	};


protected:
	// camera parameters
	GLdouble		nearPlane,
					farPlane,
					d,
					phi,
					theta,
					psi,
					fov;

	// the object being followed
	_Object3		*pTarget;

	// camera direction vector (+ mutex), view plane normal vectors
	SDLMutex		mUpdate;
	Vector			vDirection;
	Vector4F		vPlanes[6];


	void	_getPlanes();


public:
	Camera();
	virtual ~Camera();


	// camera parameters
	void			planes(const GLdouble &_nearPlane, const GLdouble &_farPlane);

	inline const GLdouble &	getNearPlane() const
	{	return nearPlane;	}
	inline const GLdouble &	getFarPlane() const
	{	return farPlane;	}


	// camera view
	inline void		distance(GLdouble _d)
	{	d = _d;	}
	inline void		increaseDistance(const GLdouble &_d)
	{	distance(d + _d);	}

	inline GLdouble	getDistance() const
	{	return d;	}

	void					rotate(const GLdouble &_phi, const GLdouble &_theta);
	inline void				rotateBy(const GLdouble &_phi, const GLdouble &_theta)
	{	rotate(phi + _phi, theta + _theta);	}

	inline const GLdouble &	getPhi() const
	{	return phi;	}
	inline const GLdouble &	getTheta() const
	{	return theta;	}

	// return the direction the the camera points to
	Vector					getDirection();
	VectorF					getDirectionF();
	// return the camera position
	Vector					getPosition();
	VectorF					getPositionF();


	void					roll(const GLdouble &_psi);
	inline void				rollBy(const GLdouble &_psi)
	{	roll(psi + _psi);	}

	inline const GLdouble &	getPsi() const
	{	return psi;	}


	void					fieldOfView(const GLdouble &_fov);

	inline const GLdouble &	getFieldOfView() const
	{	return fov;	}


	// camera target
	inline void	follow(_Object3 *_pTarget)
	{	pTarget = _pTarget;	}

	inline _Object3 *		getTarget()
	{	return pTarget;	}


	// visibility check
	Visibility	visibility(_Object3 *pObject);


	// rendering
	void	glLook();
};


GRAPHICS_END
#endif
