#ifndef __GRAPHICS__COLOUR_H__INCLUDED__
#define __GRAPHICS__COLOUR_H__INCLUDED__

#include "graphics.h"
#include "_gl.h"


GRAPHICS_BEGIN


template <class NumberType, class Derived> class Colour
{
public:
	typedef Colour<NumberType, Derived>	_Type;

protected:
	NumberType	r,
				g,
				b,
				a;

public:
	Colour(	const NumberType &_r,
			const NumberType &_g,
			const NumberType &_b,
			const NumberType &_a = 1.0f	):
		r(_r), g(_g), b(_b), a(_a)
	{	/* nothing to do */	}
	Colour( const _Type &c) :
		r(c.r), g(c.g), b(c.b), a(c.a)
	{	/* nothing to do */	}

	inline void	_red(const NumberType &_r)
	{	r = _r;	}
	void		red(const NumberType &_r)
	{
		if(_r < (NumberType)0)
			r = (NumberType)0;
		else if(_r > Derived::getMax())
			r = Derived::getMax();
		else
			r = _r;
	}

	inline const NumberType &	getRed() const
	{	return r;	}

	inline void	_green(const NumberType &_g)
	{	g = _g;	}
	void		green(const NumberType &_g)
	{
		if(_g < (NumberType)0)
			g = (NumberType)0;
		else if(_g > Derived::getMax())
			g = Derived::getMax();
		else
			g = _g;
	}

	inline const NumberType &	getGreen() const
	{	return g;	}

	inline void	_blue(const NumberType &_b)
	{	b = _b;	}
	void		blue(const NumberType &_b)
	{
		if(_b < (NumberType)0)
			b = (NumberType)0;
		else if(_b > Derived::getMax())
			b = Derived::getMax();
		else
			b = _b;
	}

	inline const NumberType &	getBlue() const
	{	return b;	}

	inline void	_alpha(const NumberType &_a)
	{	a = _a;	}
	void		alpha(const NumberType &_a)
	{
		if(_a < (NumberType)0)
			a = (NumberType)0;
		else if(_a > Derived::getMax())
			a = Derived::getMax();
		else
			a = _a;
	}

	inline const NumberType &	getAlpha() const
	{	return a;	}
};


class ByteColour :	public Colour<GLubyte, ByteColour>
{
public:
	typedef Colour<GLubyte, ByteColour>	_Base;

	inline ByteColour(	const GLubyte &r,
						const GLubyte &g,
						const GLubyte &b,
						const GLubyte &a = 255	) :
		_Base(r, g, b, a)
	{	/* nothing to do */	}
	inline ByteColour(const _Base &c) :
		_Base(c)
	{	/* nothing to do */	}

	static inline GLubyte	getMax()
	{	return 255;	}
};


class FloatColour :	public Colour<GLfloat, FloatColour>
{
public:
	typedef Colour<GLfloat, FloatColour>	_Base;

	inline FloatColour(	const GLfloat &r,
						const GLfloat &g,
						const GLfloat &b,
						const GLfloat &a = 1.0f	) :
		_Base(r, g, b, a)
	{	/* nothing to do */	}
	inline FloatColour(const _Base &c) :
		_Base(c)
	{	/* nothing to do */	}

	static inline GLfloat	getMax()
	{	return 1.0f;	}
};


class DoubleColour :	public Colour<GLdouble, DoubleColour>
{
public:
	typedef Colour<GLdouble, DoubleColour>	_Base;

	inline DoubleColour(	const GLdouble &r,
							const GLdouble &g,
							const GLdouble &b,
							const GLdouble &a = 1.0	) :
		_Base(r, g, b, a)
	{	/* nothing to do */	}
	inline DoubleColour(const _Base &c) :
		_Base(c)
	{	/* nothing to do */	}

	static inline GLdouble	getMax()
	{	return 1.0;	}
};


GRAPHICS_END
#endif
