#ifndef __GRAPHICS__FONT_H__INCLUDED__
#define __GRAPHICS__FONT_H__INCLUDED__

#include <framework/Exception.h>
#include <framework/ObjectManager.h>
#include <framework/_string.h>

#include "GlItem.h"
#include "_Freetype_Character.h"


GRAPHICS_BEGIN


class Font	: public GlItem
{
public:
	class Id
	{
	protected:
		const string	name;
		const uint_t	size;

	public:
		inline Id(const string &_name, const uint_t _size) :	name(_name),
																size(_size)
		{	/* nothing to do */	}

		inline const string &	getName() const
		{	return name;	}
		inline uint_t			getSize() const
		{	return size;	}

		inline bool	operator <(const Id &id) const
		{
			if(size < id.size)
				return true;
			else if(size == id.size)
				return name < id.name;
			return false;
		}
		inline bool	operator <=(const Id &id) const
		{
			if(size < id.size)
				return true;
			else if(size == id.size)
				return name <= id.name;
			return false;
		}
		inline bool operator ==(const Id &id) const
		{	return size == id.size && name == id.name;	}
		inline bool operator >(const Id &id) const
		{	return !operator <=(id);	}
		inline bool operator >=(const Id &id) const
		{	return !operator <(id);	}
		inline bool operator !=(const Id &id) const
		{	return !operator ==(id);	}
	};

	enum
	{
		alignXNone		= 0x0000,
		alignXLow		= 0x0001,
		alignXCenter	= 0x0002,
		alignXHigh		= 0x0004,

		alignXMask		= 0x000f
	};
	enum
	{
		alignYNone		= 0x0000,
		alignYLow		= 0x0010,
		alignYCenter	= 0x0020,
		alignYHigh		= 0x0040,

		alignYMask		= 0x00f0
	};
	enum
	{
		alignNone			= alignXNone	| alignYNone,
		alignLowLow			= alignXLow		| alignYLow,
		alignLowCenter		= alignXLow		| alignYCenter,
		alignLowHigh		= alignXLow		| alignYHigh,
		alignCenterLow		= alignXCenter	| alignYLow,
		alignCenterCenter	= alignXCenter	| alignYCenter,
		alignCenterHigh		= alignXCenter	| alignYHigh,
		alignHighLow		= alignXHigh	| alignYLow,
		alignHighCenter		= alignXHigh	| alignYCenter,
		alignHighHigh		= alignXHigh	| alignYHigh,
	};

	typedef	_Freetype_Character					Character;
	typedef	ObjectManager<Font, Id>::Reference	Reference;


protected:
	static ObjectManager<Font, Id>	manager;

	string		strFileName;	// font file name
	GLuint		nListIDBase,	// the OpenGL list base (for rendering the characters)
				nTextureID,		// the OpenGL texture ID
				nHeight,		// the font's height
				nTextureHeight,	// the texture's height (power of 2)
				nTextureWidth;	// the texture's width (power of 2)
	GLubyte		*pTexture;		// pointer to the texture (generated in create())
	Character	*chars;			// vector containing the character data


	virtual void	_load();
	virtual void	_remove();

	virtual void	_glLoad();
	virtual void	_glRemove();


public:
	Font(const Id &_id);

	static inline Reference	create(const string &name, const uint_t size)
	{	return manager[Id(name, size)];	}
	static inline void		clean()
	{	manager.clean();	}
	static inline void		clear()
	{	manager.clear();	}


	void			getDimensions(const std::string &str, uint_t &nWidth, GLfloat &nHeight);
	void			getDimensions(const std::string &str, GLfloat &nWidth, GLfloat &nHeight);

	inline GLfloat	getHeight(const std::string &str) const
	{	return getLineHeight() * (count(str, '\n') + 1);	}
	inline GLfloat	getLineHeight() const
	{	return nHeight * 1.3f;	}

	uint_t			getWidth(const std::string &str) const;
	uint_t			getLineWidth(const std::string &str) const;


	inline void		_glDrawLine(const std::string &str) const
	{	glCallLists((GLsizei)str.length(), GL_UNSIGNED_BYTE, str.c_str());	}
	void			_glDrawLine(	const std::string &str,
									const GLfloat fX,
									const GLfloat fY,
									const GLenum eAlignment = alignLowHigh	) const;
	void			_glDraw(const std::string &str) const;
	void			_glDraw(	const std::string &str,
								const GLfloat fX,
								const GLfloat fY,
								GLenum eAlignment = alignLowHigh	) const;

	virtual void	glBind();

	void			glDrawLine(const std::string &str);
	void			glDrawLine(	const std::string &str,
								const GLfloat fX,
								const GLfloat fY,
								const GLenum eAlignment = alignLowHigh	);
	void			glDraw(const std::string &str);
	void			glDraw(	const std::string &str,
							const GLfloat fX,
							const GLfloat fY,
							GLenum eAlignment = alignLowHigh	);
};


GRAPHICS_END
#endif
