#ifndef __GRAPHICS__LOCATION_H__INCLUDED__
#define __GRAPHICS__LOCATION_H__INCLUDED__

#include "_gl.h"
#include "Vector4.h"
#include "TransformationMatrix4.h"

#include "SDLMutex.h"


GRAPHICS_BEGIN


class Location
{
public:
	typedef Vector4<GLfloat>				Vector4;
	typedef Vector3<GLfloat>				Vector;
	typedef Vector3<GLdouble>				VectorD;
	typedef	TransformationMatrix4<GLfloat>	TransformationMatrix;


protected:
	SDLMutex				mUpdate,
							mLock;

	TransformationMatrix	mTransform;
	bool					bTransform;
	Vector					vForward,
							vUp;


	void		__transform();
	inline void	_transform()
	{	if(bTransform)
		{	__transform();
			bTransform = false;	}	}


public:
	inline Location()
	{	reset();	}


	// transformations
	void		reset();

	inline void	preRotate(Vector axis, const GLfloat &theta)
	{	axis.normalise();
		preRotate(axis.getX(), axis.getY(), axis.getZ(), theta);	}
	inline void	postRotate(Vector axis, const GLfloat &theta)
	{	axis.normalise();
		postRotate(axis.getX(), axis.getY(), axis.getZ(), theta);	}
	void		preRotate(const GLfloat &x, const GLfloat &y, const GLfloat &z, GLfloat theta);
	void		postRotate(const GLfloat &x, const GLfloat &y, const GLfloat &z, GLfloat theta);

	void		preRotate(GLfloat phi, GLfloat theta, GLfloat psi);
	void		postRotate(GLfloat phi, GLfloat theta, GLfloat psi);
	void		preRotateX(GLfloat phi);
	void		postRotateX(GLfloat phi);
	void		preRotateY(GLfloat theta);
	void		postRotateY(GLfloat theta);
	void		preRotateZ(GLfloat psi);
	void		postRotateZ(GLfloat psi);

	inline void	preTranslate(const Vector &vDifference)
	{	preTranslate(vDifference.getX(), vDifference.getY(), vDifference.getZ());	}
	inline void	postTranslate(const Vector &vDifference)
	{	postTranslate(vDifference.getX(), vDifference.getY(), vDifference.getZ());	}
	inline void	preTranslate(const GLfloat &x, const GLfloat &y, const GLfloat &z)
	{	mUpdate.P();
		mTransform.preTranslate(x, y, z);
		mUpdate.V();						}
	inline void	postTranslate(const GLfloat &x, const GLfloat &y, const GLfloat &z)
	{	mUpdate.P();
		mTransform.postTranslate(x, y, z);
		mUpdate.V();						}

	inline void	preScale(const GLfloat &sx, const GLfloat &sy, const GLfloat &sz)
	{	mUpdate.P();
		mTransform.preScale(sx, sy, sz);
		mUpdate.V();						}
	inline void	postScale(const GLfloat &sx, const GLfloat &sy, const GLfloat &sz)
	{	mUpdate.P();
		mTransform.postScale(sx, sy, sz);
		mUpdate.V();						}


	// position locking
	inline void				lock()
	{	mLock.P();	}
	inline void				unlock()
	{	mLock.V();	}


	// get functions
	Vector					getPosition();
	VectorD					getPositionD();
	inline GLfloat			getPositionX() const
	{	return mTransform.getElement(0, 3);	}
	inline GLfloat			getPositionY() const
	{	return mTransform.getElement(1, 3);	}
	inline GLfloat			getPositionZ() const
	{	return mTransform.getElement(2, 3);	}

	Vector			getForward();
	VectorD			getForwardD();
	Vector			getUp();
	VectorD			getUpD();
	inline Vector	getStrafe()
	{	return getForward().cross(getUp());	}
	inline VectorD	getStrafeD()
	{	return getForwardD().cross(getUpD());	}


	// OpenGL
	void	glApply();
};


GRAPHICS_END
#endif
