#ifndef __GRAPHICS__OBJECT_H__INCLUDED__
#define __GRAPHICS__OBJECT_H__INCLUDED__

#include "GlItem.h"

#include "TransformationMatrix4.h"
#include "Vector2.h"
#include "Vector4.h"


GRAPHICS_BEGIN


class Object :	public GlItem
{
public:
	typedef TransformationMatrix4<GLfloat>	TransformationMatrix;
	typedef Vector2<GLfloat>				_Vector2;
	typedef Vector2<GLdouble>				_Vector2D;
	typedef Vector3<GLfloat>				_Vector3;
	typedef Vector3<GLdouble>				_Vector3D;
	typedef Vector4<GLfloat>				_Vector4;
	typedef Vector4<GLdouble>				_Vector4D;


protected:
	// manage removing from scene
	bool		bDeleteOnRemove;

	// modeling transformations
	TransformationMatrix	*ptmParent,
							tmLocal,
							tmFinal,
							tmGL;

	// scene this object is used by
	Scene		*pScene;


	// create a new object and insert it into the scene
	void			spawn(_Object2 *pObject);
	void			spawn(_Object3 *pObject);


public:
	inline Object() :	GlItem(),
						ptmParent(NULL)
	{	reset();
		updateFinal();	}


	// delete when removing from scene?
	inline void	deleteOnRemove(bool bDelete)
	{	bDeleteOnRemove = bDelete;	}

	inline bool	getDeleteOnRemove() const
	{	return bDeleteOnRemove;	}


	inline void	scene(Scene *_pScene)
	{	pScene = _pScene;	}


	// transform a vector with the current modelling transformation
	inline _Vector3	getTransformed(const _Vector3 &v) const
	{	return tmFinal * v;	}


	// transformations
	inline void	parentTransformation(TransformationMatrix *_ptmParent)
	{	ptmParent = _ptmParent;	}

	inline void	reset()
	{	tmLocal.unity();	}


	// compute final transformation matrix
	virtual void	updateFinal();

	// advance simulation
	virtual bool	advance(float time);


	// OpenGL
	// update GL transformation matrix
	virtual void	glUpdate();
};


GRAPHICS_END
#endif
