#ifndef __GRAPHICS__PARTICLE_H__INCLUDED__
#define __GRAPHICS__PARTICLE_H__INCLUDED__

#include "Texture.h"
#include "Object3.h"


GRAPHICS_BEGIN


// Das ist unsere Partikel-Klasse
class Particle : public Object3
{
protected:
	// Die Bewegungsrichtung des Partikels.
	Vector	m_vec_movementdirection;

	// Ob ein Partikel der Physik unterliegt, 
	// d.h. ob es sich aufgrund von Einflssen bewegt (Ja/Nein).
	bool	m_bo_physic;
	
	// Aktuelle Lebensspanne des Partikels.
	float	m_fl_life;
	
	// Ausblendungsgeschwindigkeit.
	float	m_fl_fade;
	
	// Farbe des Partikels.
	float	m_fl_r,
			m_fl_g,
			m_fl_b;	
	
	// Gre des Partikels (umso grer, desto trger verhlt es sich).
	float	m_fl_size;

	// Wachstumsrate des Partikels.
	float	m_fl_growthRate;

	// Masse des Partikels.
	float	m_fl_weight;

	// Die Textur der Pratikel.
	Texture::Reference m_tex_particletexture;

	float	phi,
			theta;


	virtual void	_load();
	virtual void	_remove();

	virtual void	_glLoad();
	virtual void	_glRemove();

	virtual void	_glRender();

public:
	Particle(const Particle &p);
	// Der Partikel-Konstruktor mit bergabeparameter.
	Particle(bool physic, float size, float growthRate, float weight, float fade, float red, float green, float blue);
	// Der Partikel-Konstruktor, ohne Farbewerte bei den bergabeparametern.
	Particle(bool physic, float size, float growthRate, float weight, float fade);
	// Der Destruktor der Klasse Partikel.
	~Particle();


	// Diese Funktionen ndern private Particle-Daten.
	inline void	movementDirection(const Vector &v)
	{	m_vec_movementdirection = v;	}
	inline void movementDirection(float x, float y, float z) 
	{	m_vec_movementdirection.set(x, y, z);	}
	inline void size(float size)
	{	m_fl_size = size;	boundingSphere(m_fl_size);	}
	inline void weight(float weight)
	{	m_fl_weight = weight;	}
	inline void texture(Texture::Reference newtexture)
	{	m_tex_particletexture = newtexture;	}
	

	// Aktualisiert die Blickrichtung und Position des Partikels.
	virtual void cameraPosition(const Vector &camera);


	virtual bool	advance(float time);


	virtual void	glReload();
};


GRAPHICS_END
#endif
