#ifndef __GRAPHICS__GLAPPLICATION_H__INCLUDED__
#define __GRAPHICS__GLAPPLICATION_H__INCLUDED__

//*** include files
//*****************
#include <list>
#include <string>

#include <SDL.h>

#include <framework/Application.h>

#include "SDLThread.h"
#include "SDLMutex.h"

#include "Scene.h"

#define SDL_EVENT_BUFFER_SIZE	16


GRAPHICS_BEGIN


DEFINE_EXCEPTION(GlewException, Exception, "GLEW Exception");


//*** class definition
//********************
class SDLApplication : public Application
{
protected:
	// protected member variables
	framework::BOOL		bContinue,
						bGlewInitialised;

	// scene graph
	SDLMutex			mScene;
	Scene				*pScene;

	// video modes
	SDL_Surface			*sdlSurface;
	string				strCaption;
	framework::BOOL		bUpdateVideoMode,
						bFullscreen,
						bToggleFullscreen,
						bHaveFocus;
	uint_t				nHeight,
						nWidth,
						nBPP,
						nIgnoreNextMouseMoves;

	// message stuff
	void			_dispatchEvent(SDL_Event &event);

	// message callback functions
	virtual void	onActivate(SDL_ActiveEvent &ae);

	virtual void	onKeyDown(SDL_keysym &keysym);
	virtual void	onKeyUp(SDL_keysym &keysym);

	virtual void	onMouseMove(SDL_MouseMotionEvent &mme);
	virtual void	onMouseDown(SDL_MouseButtonEvent &mbe);
	virtual void	onMouseUp(SDL_MouseButtonEvent &mbe);

	virtual void	onJoystickAxis(SDL_JoyAxisEvent &jae);
	virtual void	onJoystickTrackball(SDL_JoyBallEvent &jbe);
	virtual void	onJoystickHat(SDL_JoyHatEvent &jhe);
	virtual void	onJoystickDown(SDL_JoyButtonEvent &jbe);
	virtual void	onJoystickUp(SDL_JoyButtonEvent &jbe);

	virtual void	onWindowManagerEvent(SDL_SysWMEvent &swme);

	virtual void	onResize(SDL_ResizeEvent &re);
	virtual void	onRedraw(SDL_ExposeEvent &ee);

	virtual void	onUserEvent(SDL_UserEvent &ue);

	virtual void	onExit();

	void			_init_glew();
	virtual void	_init(int_t argc, char *argv[]);

	void			_setVideoMode();
	void			_toggleFullscreen();

public:
	SDLApplication(const string &strCaption = "SDL Application");
	virtual ~SDLApplication();

	virtual int_t	run();
	void			stop()
	{	bContinue = FALSE;	}

	virtual void	exit(int_t nCode = 0);

	void			setScene(Scene *_pScene);

	void			setVideoMode(uint_t nWidth = 1024, uint_t nHeight = 768, framework::BOOL _bFullscreen = FALSE);
	void			toggleFullscreen()
	{	bToggleFullscreen = TRUE;	}
};


GRAPHICS_END
#endif
