#ifndef __GRAPHICS__SDL_MUTEX_H__INCLUDED__
#define __GRAPHICS__SDL_MUTEX_H__INCLUDED__

#include <SDL_mutex.h>

#include "SDLException.h"


GRAPHICS_BEGIN


class SDLMutex
{
public:
	class Allocator :
		public __AllocatorBase,
		public _StaticAllocatorBase<Allocator>
	{
	public:
		SDL_mutex *	_new() const;
		void		_delete(SDL_mutex *sdlMutex) const;
	};

protected:
	static Allocator							allocator;
	Reference<SDL_mutex, void, Allocator, true>	sdlMutex;

public:
	inline SDLMutex() :	sdlMutex(&allocator)
	{	/* nothing to do */	}
	inline SDLMutex(SDLMutex &m)	: sdlMutex(m.sdlMutex)
	{	/* nothing to do */	}
	virtual ~SDLMutex();

	inline SDLMutex &	operator =(SDLMutex &m)
	{
		if(&m != this)
			sdlMutex = m.sdlMutex;
		return *this;
	}

	void	P();
	void	V();

	inline void	lock()
	{	P();	}
	inline void	unlock()
	{	V();	}

	inline operator SDL_mutex *()
	{	return sdlMutex.getPointer();	}
	inline operator const SDL_mutex *() const
	{	return sdlMutex.getPointer();	}
};


GRAPHICS_END
#endif
