#ifndef __GRAPHICS__VECTOR_4_H__INCLUDED__
#define __GRAPHICS__VECTOR_4_H__INCLUDED__

#include <math.h>

#include "Vector3.h"


GRAPHICS_BEGIN


template <class NumberType> class Vector4 :	public Matrix<NumberType, 4, 1>
{
public:
	typedef Matrix<NumberType, 4, 1>	_Base;
	typedef Vector4<NumberType>			_Type;

	inline Vector4<NumberType>()
	{	/* nothing to do */	}
	inline Vector4<NumberType>(NumberType *pVector) :		_Base(pVector)
	{	/* nothing to do */	}
	inline Vector4<NumberType>(const NumberType *pVector) :	_Base(pVector)
	{	/* nothing to do */	}
	inline Vector4<NumberType>(const _TopBase &vector) :	_Base(vector)
	{	/* nothing to do */	}
	inline Vector4<NumberType>(const Vector3<NumberType> &vector)
	{
		_element(0) = vector.getX();
		_element(1) = vector.getY();
		_element(2) = vector.getZ();
		_element(3) = 1.0f;
	}
	inline Vector4<NumberType>(	const NumberType &_x,
								const NumberType &_y,
								const NumberType &_z,
								const NumberType &_w = 1.0f	)
	{
		_element(0) = _x;
		_element(1) = _y;
		_element(2) = _z;
		_element(3) = _w;
	}

	inline _Type &	operator =(NumberType *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const NumberType *pVector)
	{	_Base::operator =(pVector);
		return *this;				}
	inline _Type &	operator =(const _TopBase &vector)
	{	_Base::operator =(vector);
		return *this;				}
	_Type & operator =(const Matrix<NumberType, 3, 1> &vector)
	{
		element(0) = vector.getX();
		_element(1) = vector.getY();
		_element(2) = vector.getZ();
		_element(3) = 1.0f;

		return *this;
	}

	inline NumberType &			x()
	{	return element(0);	}
	inline const NumberType &	getX() const
	{	return getElement(0);	}
	inline NumberType &			y()
	{	return element(1);	}
	inline const NumberType &	getY() const
	{	return getElement(1);	}
	inline NumberType &			z()
	{	return element(2);	}
	inline const NumberType &	getZ() const
	{	return getElement(2);	}
	inline NumberType &			w()
	{	return element(3);	}
	inline const NumberType &	getW() const
	{	return getElement(3);	}

	void set(	const NumberType &_x,
				const NumberType &_y,
				const NumberType &_z,
				const NumberType &_w = 1.0f	)
	{
		element(0) = _x;
		_element(1) = _y;
		_element(2) = _z;
		_element(3) = _w;
	}

	inline NumberType			magnitude3Squared() const
	{	return getX() * getX() + getY() * getY() + getZ() * getZ();	}
	inline NumberType			magnitude3() const
	{	return sqrt(magnitude3Squared());	}

	inline _Type &				normalise3()
	{	operator /=(magnitude3());
		return *this;			}
	void						getNormalised3(_Type &normalised) const
	{	normalised /= magnitude3();	}
	_Type						getNormalised3() const
	{
		_Type	normalised;
		getNormalised3(normalised);
		return normalised;
	}

	inline NumberType			magnitudeSquared() const
	{	return getW() * getW() + getX() * getX() + getY() * getY() + getZ() * getZ();	}
	inline NumberType			magnitude() const
	{	return sqrt(magnitudeSquared());	}

	inline _Type &				normalise()
	{	operator /=(magnitude());
		return *this;			}
	void						getNormalised(_Type &normalised) const
	{	normalised /= magnitude();	}
	_Type						getNormalised() const
	{
		_Type	normalised;
		getNormalised(normalised);
		return normalised;
	}

	inline _Type &				cross(const _TopBase &vector)
	{	operator =(getCrossed(vector));
		return *this;					}
	void						getCrossed(_TopBase &crossed, const _TopBase &vector) const
	{
		crossed.element(0) = getY() * vector.getElement(2) - vector.getElement(1) * getZ();
		crossed._element(1) = getZ() * vector.getElement(0) - vector.getElement(2) * getX();
		crossed._element(2) = getX() * vector.getElement(1) - vector.getElement(0) * getY();
		crossed._element(3) = 1.0f;
	}
	inline _Type				getCrossed(const _TopBase &vector) const
	{
		_Type	crossed;
		getCrossed(crossed, vector);
		return crossed;
	}

	NumberType					dot(const Vector3<NumberType> &vector) const
	{
		return	getElement(0) * vector.getX() +
				getElement(1) * vector.getY() +
				getElement(2) * vector.getZ() +
				getElement(3);
	}
	NumberType					dot(const _Type &vector) const
	{
		return	getElement(0) * vector.getElement(0) +
				getElement(1) * vector.getElement(1) +
				getElement(2) * vector.getElement(2) +
				getElement(3) * vector.getElement(3);
	}

	inline _Type &				multiplyLeft(const Matrix<NumberType, 3, 3> &matrix)
	{	*this = matrix * (*this);
		return *this;				}
	inline _Type &				multiplyLeft(const Matrix<NumberType, 4, 4> &matrix)
	{	*this = matrix * (*this);
		return *this;				}

	inline NumberType &			operator[](uint_t index)
	{	return element(index);	}
	inline const NumberType &	operator[](uint_t index) const
	{	return getElement(index);	}

	inline operator Vector3<NumberType>() const
	{	return Vector3<NumberType>(getX(), getY(), getZ());	}
};

template <class NumberType> Vector4<NumberType>			operator *(const Matrix<NumberType, 3, 3> &matrix, const Vector4<NumberType> &vector)
{
	return Vector4<NumberType>(	vector.getX() * matrix.getElement(0, 0) + vector.getY() * matrix.getElement(0, 1) + vector.getZ() * matrix.getElement(0, 2),
								vector.getX() * matrix.getElement(1, 0) + vector.getY() * matrix.getElement(1, 1) + vector.getZ() * matrix.getElement(1, 2),
								vector.getX() * matrix.getElement(2, 0) + vector.getY() * matrix.getElement(2, 1) + vector.getZ() * matrix.getElement(2, 2)	);
}


GRAPHICS_END
#endif
