#ifndef __GRAPHICS___FREETYPE_H__INCLUDED__
#define __GRAPHICS___FREETYPE_H__INCLUDED__

#include <map>

#include <ft2build.h>
#include <freetype/freetype.h>
#include <freetype/ftglyph.h>

#include <framework/Exception.h>

#include "SDLMutex.h"

#include "_Freetype_Character.h"

#define	getFreetypeError(error)\
	(string(__FUNCTION__) + ": " + g_freetype.getError(error))
#define	getFreetypeErrorMessage(message, error)\
	(string(__FUNCTION__) + ": " + (message) + ": " + g_freetype.getError(error))


GRAPHICS_BEGIN


DEFINE_EXCEPTION(FreetypeException, Exception, "Freetype Exception");


class _Freetype
{
public:
	typedef	_Freetype_Character	Character;


protected:
	FT_Library			libFreetype;
	map<int_t, string>	mErrors;
	SDLMutex			mLock;

	void	init();


public:
	inline _Freetype()
	{	init();	}
	inline ~_Freetype()
	{	FT_Done_FreeType(libFreetype);	}

	const string &	getError(const FT_Error &code);

	FT_Face		loadFont(const string &name);
	void		setFontSize(FT_Face &face, const uint_t size);
	inline void	freeFont(FT_Face &face)
	{	FT_Done_Face(face);	}

	void	loadGlyphs(FT_Face &face, FT_BitmapGlyph *glyphs, Character *chars);
	void	freeGlyphs(FT_BitmapGlyph *glyphs);

	inline operator FT_Library()
	{	return libFreetype;	}
};


extern _Freetype	g_freetype;


GRAPHICS_END
#endif
