#include <framework/_windows.h>

#include <aquawars/Simulation.h>
#include <aquawars/VehicleAI.h>


void Simulation::initialise()
{
	Sound::initialise();
	Projectile::addSounds();

	env.scene(this);
	env.addEnvironment(Vector(-2500.0f, 0.0f, -2500.0f), 0.0f, "res/sounds/music0.mp3", 64, 0.05f, 0.3f, 0.1f, 0.0001f, 15.0f, 600.0f, 0.15f, 0.9f, 0.3f);
	env.addEnvironment(Vector(-2000.0f, 0.0f, 2500.0f), 0.0f, "res/sounds/music1.mp3", 64, 0.3f, 0.3f, 0.05f, 0.000075f, 15.0f, 600.0f, 0.9f, 0.9f, 0.15f);
	env.addEnvironment(Vector(2000.0f, 0.0f, -1700.0f), 0.0f, "res/sounds/music2.mp3", 64, 0.05f, 0.1f, 0.3f, 0.0001f, 15.0f, 600.0f, 0.15f, 0.3f, 0.9f);
	env.addEnvironment(Vector(2500.0f, 0.0f, 2000.0f), 0.0f, "res/sounds/music3.mp3", 64, 0.3f, 0.1f, 0.05f, 0.000075f, 15.0f, 600.0f, 0.6f, 0.2f, 0.1f);

	setLight(0.8f, 0.8f, 0.8f, 1.0f, 0.0f, -0.85f, 0.527f);
	heightMap.setLightVector(0.0f, -0.85f, 0.527f, 0.4f);

	camera.planes(0.1, 1500.0f);

	camera.follow(&sub);
	camera.distance(15.0);
	camera.rotate(PI / -16.0, 0.0);

	setCamera(&camera);
}

void Simulation::load()
{
	ParticleList::explosionTexture(Texture::create("res/textures/particles/explosion.tga"));
	ParticleList::flashTexture(Texture::create("res/textures/particles/lichtblitz.tga"));
	ParticleList::bubbleTexture(Texture::create("res/textures/particles/blase.tga"));
	ParticleList::sparkTexture(Texture::create("res/textures/particles/funke.tga"));
	ParticleList::cloudTexture(Texture::create("res/textures/particles/cloud.tga"));

	Plant::texture(0, Texture::create("res/textures/plant00.tga"));
	Plant::texture(1, Texture::create("res/textures/plant01.tga"));
	Plant::texture(2, Texture::create("res/textures/plant02.tga"));
	Plant::texture(3, Texture::create("res/textures/plant03.tga"));
	Plant::texture(4, Texture::create("res/textures/plant04.tga"));
	Plant::texture(5, Texture::create("res/textures/plant05.tga"));
	Plant::texture(6, Texture::create("res/textures/plant06.tga"));
	Plant::texture(7, Texture::create("res/textures/plant07.tga"));
	Plant::texture(8, Texture::create("res/textures/plant08.tga"));
	Plant::texture(9, Texture::create("res/textures/plant09.tga"));
}


void Simulation::_load_level()
{
	setFog(0.0f, 0.0f, 0.0f, 1.0f, 0.001f, 1.0f, GL_LINEAR);

	// player model + UI
	sub.load();
	sub.reset();
	sub.postTranslate(-2100.0f, 300.0f, -2100.0f);
	//sub.postTranslate(1750.0f, 420.0f, -500.0f);
	sub.updateFinal();

	sub.addWeapon(Weapon::LightMG);	
	sub.addWeapon(Weapon::LightCannon);

	sub.scene(this);
	add(&sub);

	hud.font(Font::create("res/tahoma.ttf", 12));
	hud.skin("res/textures/HUD");
	
	hud.load();
	hud.vehicle(&sub);
	hud.scene(this);
	add(&hud);

	heightMap.textures(Texture::create("res/textures/maps/sand.tga"), Texture::create("res/textures/maps/rock.tga"));
	heightMap.load();
	add(heightMap);

	// 1. Stadt
	add(Plant::createPlants(Vector(-2000.0f, 0.0f, -2000.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-2000.0f, 0.0f, -2000.0f), 8, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1700.0f, 0.0f, -2200.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1700.0f, 0.0f, -2200.0f), 8, 15.0f, 125.0f, 25, heightMap));

	// Plantagen
	for(float i = -1400; i > -1900; i-=200)
		for(float j = -800; j > -1900; j-=200)
			add(Plant::createField(Vector(i, 0.0f, j), 1, 15.0f, 70.0f, 70.0f, 45, heightMap));

	// Eingang zur 2. Etappe
	add(Plant::createPlants(Vector(-1100.0f, 0.0f, -700.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1100.0f, 0.0f, -600.0f), 8, 15.0f, 125.0f, 25, heightMap));
	
	add(Plant::createPlants(Vector(-1500.0f, 0.0f, -700.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1500.0f, 0.0f, -600.0f), 8, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1300.0f, 0.0f, -500.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1300.0f, 0.0f, -400.0f), 8, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1300.0f, 0.0f, -300.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1300.0f, 0.0f, -350.0f), 8, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1300.0f, 0.0f, -200.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1300.0f, 0.0f, -100.0f), 8, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1300.0f, 0.0f, 0.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1800.0f, 0.0f, 400.0f), 8, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1900.0f, 0.0f, 400.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1300.0f, 0.0f, 300.0f), 8, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1300.0f, 0.0f, 350.0f), 2, 15.0f, 125.0f, 25, heightMap));

	// 2. Etappe
	add(Plant::createPlants(Vector(-1600.0f, 0.0f, 1000.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1700.0f, 0.0f, 1300.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1600.0f, 0.0f, 1000.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1700.0f, 0.0f, 1300.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1900.0f, 0.0f, 1600.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-2000.0f, 0.0f, 1900.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1600.0f, 0.0f, 1700.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1700.0f, 0.0f, 2000.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-1000.0f, 0.0f, 1700.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-1000.0f, 0.0f, 2000.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-900.0f, 0.0f, 2000.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-700.0f, 0.0f, 1700.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-600.0f, 0.0f, 1600.0f), 0, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-400.0f, 0.0f, 1400.0f), 2, 15.0f, 125.0f, 25, heightMap));

	// Ausgang der 2. Etappe
	add(Plant::createPlants(Vector(-450.0f, 0.0f, 400.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-450.0f, 0.0f, 600.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-50.0f, 0.0f, 400.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-50.0f, 0.0f, 600.0f), 2, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(50.0f, 0.0f, 200.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(100.0f, 0.0f, 100.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-50.0f, 0.0f, 0.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-150.0f, 0.0f, 100.0f), 9, 15.0f, 125.0f, 25, heightMap));

	// 3. Etappe
	add(Plant::createPlants(Vector(-100.0f, 0.0f, 0.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-200.0f, 0.0f, -100.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-100.0f, 0.0f, -300.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-200.0f, 0.0f,-400.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(-100.0f, 0.0f, -500.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-200.0f, 0.0f,-700.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(0.0f, 0.0f, -900.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-100.0f, 0.0f, -1100.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(0.0f, 0.0f, -1300.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-100.0f, 0.0f,-1500.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(0.0f, 0.0f, -1700.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-100.0f, 0.0f,-1900.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(500.0f, 0.0f, 0.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(400.0f, 0.0f, -200.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(500.0f, 0.0f, -400.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(400.0f, 0.0f, -600.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(500.0f, 0.0f, -500.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(400.0f, 0.0f, -700.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(500.0f, 0.0f, -800.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(400.0f, 0.0f, -900.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(500.0f, 0.0f, -1000.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(400.0f, 0.0f, -1200.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(800.0f, 0.0f, -1200.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(900.0f, 0.0f, -1300.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1200.0f, 0.0f, -1200.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1400.0f, 0.0f, -1300.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1400.0f, 0.0f, -1000.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1300.0f, 0.0f, -800.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1300.0f, 0.0f, -600.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1300.0f, 0.0f, -500.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1300.0f, 0.0f, -400.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1600.0f, 0.0f, -200.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(200.0f, 0.0f, -1300.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(-100.0f, 0.0f,-1500.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(300.0f, 0.0f, -1700.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(500.0f, 0.0f, -1900.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(700.0f, 0.0f, -1900.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(900.0f, 0.0f, -2100.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(800.0f, 0.0f, -1900.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1000.0f, 0.0f,-2100.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1000.0f, 0.0f, -1900.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1200.0f, 0.0f, -2100.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1400.0f, 0.0f, -1900.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1600.0f, 0.0f, -2100.0f), 4, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1900.0f, 0.0f, -1500.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1700.0f, 0.0f, -1700.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1900.0f, 0.0f, -1300.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1700.0f, 0.0f, -1100.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1900.0f, 0.0f, -900.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1700.0f, 0.0f, -800.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(2000.0f, 0.0f, -600.0f), 4, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1800.0f, 0.0f, -500.0f), 9, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(2200.0f, 0.0f, -450.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(2000.0f, 0.0f, -250.0f), 4, 15.0f, 125.0f, 25, heightMap));

	// Verbindungsgang der 3. zu 4. Etappe
	add(Plant::createPlants(Vector(1950.0f, 0.0f, 0.0f), 9, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1950.0f, 0.0f, 200.0f), 8, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1950.0f, 0.0f, 0.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1950.0f, 0.0f, 200.0f), 8, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1950.0f, 0.0f, 400.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(1950.0f, 0.0f, 600.0f), 8, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(1950.0f, 0.0f, 800.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(750.0f, 0.0f, 1000.0f), 8, 20.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(650.0f, 0.0f, 1000.0f), 7, 15.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(650.0f, 0.0f, 1200.0f), 7, 15.0f, 125.0f, 25, heightMap));

	add(Plant::createPlants(Vector(650.0f, 0.0f, 1300.0f), 8, 20.0f, 125.0f, 25, heightMap));
	add(Plant::createPlants(Vector(650.0f, 0.0f, 1400.0f), 7, 15.0f, 125.0f, 25, heightMap));

	// 4. Etappe
	add(Plant::createPlants(Vector(2100.0f, 0.0f, 1400.0f), 0, 25.0f, 150.0f, 15, heightMap));
	add(Plant::createPlants(Vector(2100.0f, 0.0f, 1750.0f), 7, 20.0f, 150.0f, 20, heightMap));
	
	add(Plant::createPlants(Vector(2100.0f, 0.0f, 2100.0f), 0, 25.0f, 150.0f, 15, heightMap));
	add(Plant::createPlants(Vector(1750.0f, 0.0f, 1400.0f), 7, 15.0f, 150.0f, 20, heightMap));
	
	add(Plant::createPlants(Vector(1600.0f, 0.0f, 1400.0f), 0, 25.0f, 150.0f, 15, heightMap));
	add(Plant::createPlants(Vector(1400.0f, 0.0f, 2100.0f), 0, 25.0f, 150.0f, 15, heightMap));

	add(Plant::createPlants(Vector(1800.0f, 0.0f, 2100.0f), 7, 20.0f, 150.0f, 20, heightMap));	

	Sunray::position(sub.getPosition());
	Sunray::direction(Vector3<float>(0.0f, -0.85f, 0.527f));
	Sunray::depth(700.0f);
	Sunray::length(400.0f);
	Sunray::turbolence(1.0f);
	Sunray::texture(Texture::create("res/textures/sunray00.tga"));
	add(Sunray::create(100));


	// "town" models
	dome1.load();
	dome1.reset();
	dome1.preScale(100.0f, 100.0f, 100.0f);
	dome1.preRotateX(-(float)PI / 2.0f);
	dome1.postTranslate(-1900.0f, 300.0f, -2200.0f);
	dome1.updateFinal();
	add(&dome1);

	dome2.load();
	dome2.reset();
	dome2.preScale(30.0f, 30.0f, 30.0f);
	dome2.preRotateX(-(float)PI / 2.0f);
	dome2.postTranslate(-2000.0f, 285.0f, -2250.0f);
	dome2.updateFinal();
	add(&dome2);

	dome3.load();
	dome3.reset();
	dome3.preScale(100.0f, 100.0f, 100.0f);
	dome3.preRotateX(-(float)PI / 2.0f);
	dome3.postTranslate(100.0f, 270.0f, -200.0f);
	dome3.updateFinal();
	add(&dome3);

	dome4.load();
	dome4.reset();
	dome4.preScale(100.0f, 100.0f, 100.0f);
	dome4.preRotateX(-(float)PI / 2.0f);
	dome4.postTranslate(300.0f, 320.0f, -850.0f);
	dome4.updateFinal();
	add(&dome4);

	dome5.load();
	dome5.reset();
	dome5.preScale(200.0f, 200.0f, 200.0f);
	dome5.preRotateX(-(float)PI / 2.0f);
	dome5.postTranslate(1700.0f, 350.0f, -1500.0f);
	dome5.updateFinal();
	add(&dome5);

	dome6.load();
	dome6.reset();
	dome6.preScale(30.0f, 30.0f, 30.0f);
	dome6.preRotateX(-(float)PI / 2.0f);
	dome6.postTranslate(1950.0f, 320.0f, -300.0f);
	dome6.updateFinal();
	add(&dome6);

	dome7.load();
	dome7.reset();
	dome7.preScale(30.0f, 30.0f, 30.0f);
	dome7.preRotateX(-(float)PI / 2.0f);
	dome7.postTranslate(1780.0f, 305.0f, -220.0f);
	dome7.updateFinal();
	add(&dome7);

	harvester1.load();
	harvester1.reset();
	harvester1.preScale(15.0f, 15.0f, 15.0f);
	harvester1.postRotateX(-(float)PI / 2.0f);
	harvester1.postRotateY(-(float)PI / 3.0f);
	harvester1.postTranslate(-1970.0f, 280.0f, -2150.0f);
	harvester1.updateFinal();
	add(&harvester1);

	harvester2.load();
	harvester2.reset();
	harvester2.preScale(15.0f, 15.0f, 15.0f);
	harvester2.postRotateX(-(float)PI / 2.0f);
	harvester2.postRotateY(-(float)PI / 2.0f);
	harvester2.postTranslate(-2000.0f, 280.0f, -2200.0f);
	harvester2.updateFinal();
	add(&harvester2);

	harvester3.load();
	harvester3.reset();
	harvester3.preScale(15.0f, 15.0f, 15.0f);
	harvester3.postRotateX(-(float)PI / 2.0f);
	harvester3.postTranslate(-2030.0f, 280.0f, -2220.0f);
	harvester3.updateFinal();
	add(&harvester3);

	harvester4.load();
	harvester4.reset();
	harvester4.preScale(15.0f, 15.0f, 15.0f);
	harvester4.postRotateX(-(float)PI / 2.0f);
	harvester4.postTranslate(-1750.0f, 300.0f, 1600.0f);
	harvester4.updateFinal();
	add(&harvester4);

	harvester5.load();
	harvester5.reset();
	harvester5.preScale(15.0f, 15.0f, 15.0f);
	harvester5.postRotateX(-((float)PI / 2.0f  - (float)PI / 10.0f));
	harvester5.postRotateY(-(float)PI / 10.0f);
	harvester5.postTranslate(-1700.0f, 295.0f, 1585.0f);
	harvester5.updateFinal();
	add(&harvester5);
}


void Simulation::_start_quest(const uint_t &quest)
{
	nCurrentQuest = quest;

	switch(quest)
	{
	case 1:
		{
			// place enemies
			VehicleAI	*pEnemy;

			pEnemy = new VehicleAI("res/models/en1/en1.txt", Vector(-1750.0f, 275.0f, -1750.0f), 350.0f, 350.0f, 7.0f, 250.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1750.0f, 325.0f, -1950.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en1/en1.txt", Vector(-1750.0f, 300.0f, -1500.0f), 350.0f, 350.0f, 7.0f, 250.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1900.0f, 375.0f, -1750.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en2/en2.txt", Vector(-1600.0f, 240.0f, -1000.0f), 350.0f, 350.0f, 7.0f, 450.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1650.0f, 290.0f, -1200.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::LightMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en2/en2.txt", Vector(-1650.0f, 260.0f, -850.0f), 350.0f, 350.0f, 7.0f, 450.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1650.0f, 250.0f, -1150.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::LightMG);
			add(pEnemy);

			// reset quest counter variables
			nShipsDestroyed = 0;

			// send starting message
			hud.message("Gott sei dank bist du gekommen!\n"
						"Unsere Stadt wird seit Wochen von den\n"
						"Luzisten belagert und sie bedrohen unsere\n"
						"Ernte. Bitte hilf uns sie zu vertreiben!",
						Texture::create("res/textures/HUD/transmission/boss.tga")	);

			break;
		}

	case 2:
		{
			// place enemies
			VehicleAI	*pEnemy;

			pEnemy = new VehicleAI("res/models/en2/en2.txt", Vector(-1723.0f, 297.5f, 1592.5f), 350.0f, 350.0f, 7.0f, 550.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1723.0f, 280.0f, 1692.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en3/en3.txt", Vector(-1723.0f, 297.5f, 1592.5f), 350.0f, 350.0f, 7.0f, 550.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1723.0f, 380.0f, 1600.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::LightCannon);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en2/en2.txt", Vector(-900.0f, 360.5f, 1592.5f), 350.0f, 350.0f, 7.0f, 550.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-900.0f, 370.5f, 1600.5f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en3/en3.txt", Vector(-1000.0f, 330.5f, 1600.0f), 350.0f, 350.0f, 7.0f, 550.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(-1000.0f, 350.5f, 1650.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::LightCannon);
			add(pEnemy);

			// reset quest counter variables
			nShipsDestroyed = 0;

			// send starting message
			hud.message("Vielen Dank... unsere Ernte ist gerettet!\n"
						"Doch zwei unserer Ernstemaschinen sind\n" 
						"in einen Hinterhalt der Luszisten\n" 
						"geraten. Ohne Schutz sind sie verloren!",
						Texture::create("res/textures/HUD/transmission/boss.tga")	);

			hud.message("Die Erntemaschinen befinden sich in den\n"
						"sdlichen Schluchten. Wir stellen Dir\n"
						"verbesserte Waffensysteme zur Verfgung.\n"
						"Beeile Dich bitte und viel Glck!",
						Texture::create("res/textures/HUD/transmission/boss.tga")	);

			sub.addWeapon(Weapon::HeavyMG);
			sub.addWeapon(Weapon::HeavyCannon);
			

			break;
		}

	case 3:
		{
			// place enemies
			VehicleAI	*pEnemy;

			pEnemy = new VehicleAI("res/models/en3/en3.txt", Vector(180.0f, 370.0f, -300.0f), 350.0f, 350.0f, 7.0f, 650.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(150.0f, 350.0f, -300.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(250.0f, 350.0f, -700.0f), 350.0f, 350.0f, 7.0f, 650.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(200.0f, 370.0f, -650.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::LightCannon);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en3/en3.txt", Vector(1500.0f, 400.0f, -1250.0f), 350.0f, 350.0f, 7.0f, 650.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1550.0f, 350.0f, -1250.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(1750.0f, 420.0f, -850.0f), 350.0f, 350.0f, 7.0f, 650.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1650.0f, 400.0f, -850.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(1750.0f, 420.0f, -500.0f), 350.0f, 350.0f, 7.0f, 650.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1800.0f, 420.0f, -450.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::LightCannon);
			add(pEnemy);

			// reset quest counter variables
			nShipsDestroyed = 0;

			// send starting message
			hud.message("Danke fr die Hilfe! Wir waren gerade auf\n"
						"dem Weg zu den Stdten der Hochebene,\n"
						"als wir in diesen Hinterhalt geraten sind.",
						Texture::create("res/textures/HUD/transmission/harvester.tga")	);

			hud.message("Leider war das aber nur die Nachthut einer\n"
						"einer groen Offensive gegen die Stdte.\n"
						"Sie wollen einen Putsch versuchen. Nur Du\n"
						"stehst noch zwischen ihnen und dem Senat!",
						Texture::create("res/textures/HUD/transmission/harvester.tga")	);

			sub.addWeapon(Weapon::TorpedoLauncher);

			break;
		}

	case 4:
		{
			// place enemies
			VehicleAI	*pEnemy;	

			pEnemy = new VehicleAI("res/models/en3/en3.txt", Vector(1900.0f, 250.0f, 1750.0f), 350.0f, 350.0f, 7.0f, 800.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1850.0f, 255.0f, 1750.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyCannon);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(1750.0f, 300.0f, 1500.0f), 350.0f, 350.0f, 7.0f, 800.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1750.0f, 390.0f, 1440.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en3/en3.txt", Vector(1800.0f, 190.0f, 2000.0f), 350.0f, 350.0f, 7.0f, 800.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1750.0f, 200.0f, 2000.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyCannon);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(1500.0f, 220.0f, 1850.0f), 350.0f, 350.0f, 7.0f, 800.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1550.0f, 200.0f, 1750.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyMG);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(1450.0f, 230.0f, 1550.0f), 350.0f, 350.0f, 7.0f, 800.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1450.0f, 250.0f, 1450.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyCannon);
			add(pEnemy);

			pEnemy = new VehicleAI("res/models/en4/en4.txt", Vector(1400.0f, 320.0f, 1750.0f), 350.0f, 350.0f, 7.0f, 800.0f);
			pEnemy->deleteOnRemove(true);
			pEnemy->load();
			pEnemy->reset();
			pEnemy->postTranslate(1400.0f, 300.0f, 1700.0f);
			pEnemy->updateFinal();
			pEnemy->scene(this);
			pEnemy->addWeapon(Weapon::HeavyCannon);
			add(pEnemy);

			// reset quest counter variables
			nShipsDestroyed = 0;

			// send starting message
			hud.message("Der Senat bedankt sich fr deine\n"
						"Untersttzung. Die Luzisten sind aber\n"
						"noch nicht besiegt. Sie fliehen in die\n"
						"Hhlensysteme.",
						Texture::create("res/textures/HUD/transmission/storm.tga")	);

			hud.message("Dort hast du eine Chance sie zu \n"
						"vernichten! Wir rsten sie neu aus\n"
						"fr diesen letzten Auftrag...\n"
						"Viel Glck, der Senat zhlt auf Dich!",
						Texture::create("res/textures/HUD/transmission/storm.tga")	);

			sub.addWeapon(Weapon::GaussCannon);

			break;
		}
	
	case 5:
		_win();
	}

	sub.refillWeapons();
	sub.repair();
}

bool Simulation::_done_quest()
{
	switch(nCurrentQuest)
	{
	case 1:
		if(nShipsDestroyed > 3)
			return true;
		break;
	case 2:
		if(nShipsDestroyed > 3)
			return true;
		break;
	case 3:
		if(nShipsDestroyed > 4)
			return true;
		break;
	case 4:
		if(nShipsDestroyed > 5)
			return true;
		break;
	}

	return false;
}


void Simulation::_simulate_view(float fTime)
{
	// update camera
	if(distanceChange)
	{
		camera.increaseDistance(distanceChange);
		distanceChange = 0.0f;
	}
	if(phiChange || thetaChange)
	{
		camera.rotateBy(phiChange, thetaChange);
		phiChange = 0.0f;
		thetaChange = 0.0f;
	}

	if(bResetView)
	{
		fTime = pow(0.95f, fTime * 60.0f);
		camera.rotate(	- (GLfloat)PI / 8.0f + (camera.getPhi() + (GLfloat)PI / 8.0f) * fTime,
						camera.getTheta() * fTime	);
		if(fabs(camera.getPhi() + (GLfloat)PI / 8.0f) < 0.0002 && fabs(camera.getTheta()) < 0.0002)
		{
			camera.rotate(PI / -8.0, 0.0);
			bResetView = false;
		}
	}
}

void Simulation::_simulate_sub(float fTime)
{
	if(bWon || bLost)
		return;

	if(nWeapon)
	{
		sub.changeWeapon(nWeapon);
		nWeapon = 0;
	}

	if(nIncreasingPower)
	{
		fIncreasingPowerTime += fTime;

		const uint_t	nSteps = (uint_t)(fIncreasingPowerTime / fIncreasingPowerStepTime);

		fEnginePower += 0.02f * nSteps * nIncreasingPower;
		fIncreasingPowerTime -= nSteps * fIncreasingPowerStepTime;

		if(fEnginePower > 1.0f)
			fEnginePower = 1.0f;
		else if(fEnginePower < -0.2f)
			fEnginePower = -0.2f;
		else if(fEnginePower > -0.000001f && fEnginePower < 0.000001f)
			fEnginePower = 0.0f;

		sub.enginePower(fEnginePower);
	}
	sub.changeDirection(fRudderHorizontal, fRudderVertical);
	fRudderHorizontal = fRudderVertical = 0.0f;

	if(sub.getPositionY() > 550.0f)
	{
		sub.postTranslate(0.0f, 550.0f - sub.getPositionY(), 0.0f);
		sub.updateFinal();
	}

	if(sub.isDestroyed())
		_lose();
	else if(_done_quest())
		_start_quest(nCurrentQuest+1);

	Sunray::position(sub.getPosition());
}

void Simulation::simulate(float fTime)
{
	switch(gameState)
	{
	case intro1:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			Image *pImage = new Image(Texture::create("res/textures/intro1.tga"), 0.0f, 0.0f, getWidth(), getHeight(), 1.0f, 1.0f, 1.0f, 1.0f, 15.0f, 1.0f, 1.0f);
			pImage->deleteOnRemove(true);

			intro.start();
			add(pImage);
			fGameStateTime = 15.0f;
			gameState = intro2;
		}
		break;
	case intro2:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			Image *pImage = new Image(Texture::create("res/textures/intro2.tga"), 0.0f, 0.0f, getWidth(), getHeight(), 1.0f, 1.0f, 1.0f, 1.0f, 15.0f, 1.0f, 1.0f);
			pImage->deleteOnRemove(true);

			add(pImage);
			fGameStateTime = 15.0f;
			gameState = intro3;
		}
		break;
	case intro3:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			Image *pImage = new Image(Texture::create("res/textures/intro3.tga"), 0.0f, 0.0f, getWidth(), getHeight(), 1.0f, 1.0f, 1.0f, 1.0f, 15.0f, 1.0f, 1.0f);
			pImage->deleteOnRemove(true);

			add(pImage);
			fGameStateTime = 1.0f;
			gameState = init_game;
		}
		break;

	case init_game:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			_load_level();
			fGameStateTime = 14.0f;
			gameState = begin_game;
		}
		break;
	case begin_game:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			intro.stop();
			Texture::clean();

			_start_quest(1);
			gameState = playing;
		}
		break;
	case playing:
		_simulate_view(fTime);
		_simulate_sub(fTime);
		env.advance(sub.getPosition(), fTime);

		dome1.updateFinal();
		dome2.updateFinal();
		dome3.updateFinal();
		dome4.updateFinal();
		dome5.updateFinal();
		dome6.updateFinal();
		dome7.updateFinal();

		harvester1.updateFinal();
		harvester2.updateFinal();
		harvester3.updateFinal();
		harvester4.updateFinal();
		harvester5.updateFinal();

		Object3::advanceCausticTime(fTime);

		if(bWon)
		{
			gameState = outro1;
			fGameStateTime = 10.0f;
		}
		if(bLost)
		{
			gameState = end_lost;
			fGameStateTime = 30.0f;
		}
		break;

	case outro1:
		if(fGameStateTime > 0.0f)
		{
			fGameStateTime -= fTime;
			env.advance(sub.getPosition(), fTime);
		}
		else
		{
			Texture::clean();

			Image *pImage = new Image(Texture::create("res/textures/outro1.tga"), 0.0f, 0.0f, getWidth(), getHeight(), 1.0f, 1.0f, 1.0f, 1.0f, 1.1f, 1.0f, 0.0f);
			pImage->deleteOnRemove(true);

			add(pImage);
			fGameStateTime = 1.0f;
			gameState = outro1a;
		}
		break;
	case outro1a:
		if(fGameStateTime > 0.0f)
		{
			fGameStateTime -= fTime;
			env.advance(sub.getPosition(), fTime);
		}
		else
		{
			clear();

			Image *pImage = new Image(Texture::create("res/textures/outro1.tga"), 0.0f, 0.0f, getWidth(), getHeight(), 1.0f, 1.0f, 1.0f, 1.0f, 14.0f, 0.0f, 1.0f);
			pImage->deleteOnRemove(true);

			add(pImage);
			fGameStateTime = 14.0f;
			gameState = outro2;
		}
		break;
	case outro2:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			Image *pImage = new Image(Texture::create("res/textures/outro2.tga"), 0.0f, 0.0f, getWidth(), getHeight(), 1.0f, 1.0f, 1.0f, 1.0f, 5.0f, 1.0f, 1.0f);
			pImage->deleteOnRemove(true);

			add(pImage);
			fGameStateTime = 5.0f;
			gameState = end_game;
		}
		break;

	case end_lost:
		if(fGameStateTime > 0.0f)
		{
			fGameStateTime -= fTime;
			env.advance(sub.getPosition(), fTime);
		}
		else
		{
			pApplication->exit(0);
			gameState = end;
		}
		break;
	case end_game:
		if(fGameStateTime > 0.0f)
			fGameStateTime -= fTime;
		else
		{
			pApplication->exit(0);
			gameState = end;
		}
		break;
	case end:
		break;
	};
}


Simulation::Simulation() :
	pApplication(NULL),
	gameState(intro1),
	fGameStateTime(1.0f),
	nQuests(2),
	nCurrentQuest(0),
	distanceChange(0.0f),
	phiChange(0.0f),
	thetaChange(0.0f),
	fRudderHorizontal(0.0f),
	fRudderVertical(0.0f),
	bResetView(false),
	nIncreasingPower(0),
	fEnginePower(0.0f),
	fIncreasingPowerTime(0.0f),
	fIncreasingPowerStepTime(0.1f),
	speed(0.0f),
	heightMap("res/maps/level1.raw", 1024, 1024, 6, 5.0f, 5.0f / 512.0f, 5.0f),
	win("res/sounds/win.mp3"),
	lose("res/sounds/lose.mp3"),
	intro("res/sounds/intro.mp3"),
	bWon(false),
	bLost(false),
	nWeapon(0),
	sub("res/models/pl1/pl1.txt"),
	dome1("res/models/Stadt/ct_kp.txt"),
	dome2("res/models/Stadt/ct_kp.txt"),
	dome3("res/models/Stadt/ct_kp.txt"),
	dome4("res/models/Stadt/ct_kp.txt"),
	dome5("res/models/Stadt/ct_kp.txt"),
	dome6("res/models/Stadt/ct_kp.txt"),
	dome7("res/models/Stadt/ct_kp.txt"),
	harvester1("res/models/agr/agr.txt"),
	harvester2("res/models/agr/agr.txt"),
	harvester3("res/models/agr/agr.txt"),
	harvester4("res/models/agr/agr.txt"),
	harvester5("res/models/agr/agr.txt"),
	nShipsDestroyed(0)
{	/* nothing to do */	}

Simulation::~Simulation()
{
	hud.remove();
	heightMap.remove();
}


void Simulation::_win()
{
	if(bWon || bLost)
		return;

	bWon = true;
	fEnginePower = 0.0f;
	sub.enginePower(0.0f);

	hud.message("Sie haben es geschafft! Die\n"
				"Bevlkerung und der Senat ist Dir\n"
				"zu groem Dank verpflichtet! Diesen\n" 
				"glorreichen Tag werden wir nie vergessen!",
				Texture::create("res/textures/HUD/transmission/storm.tga")	);
	
	env.silent();
	win.start();
}
void Simulation::_lose()
{
	if(bWon || bLost)
		return;

	bLost = true;
	sub.enginePower(0.0f);

	hud.message("Sie haben versagt!");
	env.silent();
	lose.start();
}


void Simulation::onTargetDestroyed(const Vehicle *pVehicle)
{
	switch(rand()%6)
	{
	case 0:
		hud.message("Meine Luzisten-Brder werden mich rchen!", Texture::create("res/textures/HUD/transmission/enemy1.tga"));
		break;
	case 1:
		hud.message("Nein! Meine Systeme sind ausgefallen...\nich muss hier raaauuus...", Texture::create("res/textures/HUD/transmission/enemy1.tga"));
		break;
	case 2:
		hud.message("Ich brenne! WAAaaaahhh!", Texture::create("res/textures/HUD/transmission/enemy1.tga"));
		break;
	case 3:
		hud.message("Mein Tod wird nicht vergebens sein!", Texture::create("res/textures/HUD/transmission/enemy0.tga"));
		break;
	case 4:
		hud.message("Fr meine Gttin! Fr Luzi!", Texture::create("res/textures/HUD/transmission/enemy0.tga"));
		break;
	case 5:
		hud.message("Ich seh dich in der Hlle wieder!", Texture::create("res/textures/HUD/transmission/enemy0.tga"));
		break;
	};
	nShipsDestroyed++;
}
