#include <aquawars/VehicleAI.h>

#include <graphics/Map.h>
#include <graphics/Scene.h>


//////////////////////////// VEHICLE AI \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser VehicleAI-Konstruktor, mit bergabeparameter.
/////
//////////////////////////// VEHICLE AI \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
VehicleAI::VehicleAI(	const string &model,
						const Vector &checkpoint, const float &checkpointRadius, const float &fireRange,
						const float &_fMaxEnginePower, const float &_fMaxArmour							) :
	Vehicle(model, _fMaxEnginePower, _fMaxArmour),
	m_vec_checkpoint(checkpoint),
	m_fl_checkpointRadius(checkpointRadius),
	m_fl_fireRange(fireRange),
	m_bo_gotDamage(false)
{	beginToPatrol();	}


void VehicleAI::advance_attack(float time)
{
	if(pTarget->isDestroyed())
		beginToPatrol();

	float	distance,
			phi,
			theta;

	// Voraussichtliche Position des Ziels berechnen
	m_vec_destinationDirection.set(pTarget->getPositionX() - getPositionX(), pTarget->getPositionY() - getPositionY(), pTarget->getPositionZ() - getPositionZ());
	distance = m_vec_destinationDirection.magnitude();

	if(distance > m_fl_fireRange * 2.0f)
		beginToPatrol();

	// Ziel fr die Zeit, die unsere Schsse auf dem Weg sind nach vorne "bewegen"
	m_vec_destinationDirection += pTarget->getForward() * (distance / (fSpeed + 50.0f)) * pTarget->getSpeed();			
	m_vec_destinationDirection.normalise();
	
	getAnglesToDestination(phi, theta);
	changeDirection(theta * time, phi * time);
	
	fire((fabs(phi) <= (float)PI / 16.0f) && (fabs(theta) <= (float)PI / 16.0f));

	// Geschwindigkeit anpassen.
	if(distance < m_fl_fireRange * 0.75f)
	{
		fEnginePower -= (1.0f - distance / m_fl_fireRange) * time;
		if(distance < m_fl_fireRange * 0.3333f)
			fEnginePower = 0.0f;
	}
	else if(distance > m_fl_fireRange)
		fEnginePower += (distance / m_fl_fireRange - 1.0f) * time;

	if(fEnginePower < 0.15f)
		fEnginePower = 0.15f;
	if(fEnginePower > 1.0f)
		fEnginePower = 1.0f;
}

void VehicleAI::advance_goHome(float time)
{
	if(Vector(m_vec_checkpoint - getPosition()).magnitude() <= m_fl_checkpointRadius / 2.0f)
		beginToPatrol();
	else
	{
		m_vec_destinationDirection.set(m_vec_checkpoint.getX() - getPositionX(), m_vec_checkpoint.getY() - getPositionY(), m_vec_checkpoint.getZ() - getPositionZ());
		m_vec_destinationDirection.normalise();
		steerToDestination(time);
	}
}

void VehicleAI::advance_patrol(float time)
{
	for(list<_Object3 *>::iterator it = objects.begin(); it != objects.end(); it++)
		if((*it)->getTypeID() == player)
			beginToAttack((Vehicle *)(*it));

	if(m_state == patrol && (Vector(m_vec_checkpoint - getPosition()).magnitude() > m_fl_checkpointRadius / 2.0f))
	{
		m_vec_destinationDirection.set(m_vec_checkpoint.getX() - getPositionX(), m_vec_checkpoint.getY() - getPositionY(), m_vec_checkpoint.getZ() - getPositionZ());
		m_vec_destinationDirection.normalise();
		steerToDestination(time);
	}
}

void VehicleAI::advance_evade(float time)
{
	m_fl_stateTimer -= time;
	if(m_fl_stateTimer <= 0.0f)
		m_state = m_lastState;
	else
		steerToDestination(time);
}


VehicleAI::EvadeCause VehicleAI::haveToEvade(const Vector &vec)
{
	for(list<_Object3 *>::iterator it = objects.begin(); it != objects.end();)
		if((*it)->getCathegoryID() == c_vehicle)
		{
			Vector		v(getPosition() - (*it)->getPosition());

			const float	diff = v.magnitude();
			if(diff < fSpeed * 3)
			{
				v.normalise();

				// relativ nahe - winkel testen
				const float	a = 15.0f / diff,
							cos = fabs(v.dot(vec));

				if(cos < a)
				{
					if(cos < a / 5.0f)
						return EvadeCause::vehicleCentre;
					return EvadeCause::vehicle;
				}
			}

			it++;
		}
		else if((*it)->getCathegoryID() == c_terrain)
		{
			Map		*pMap = (Map *)((*it)->getTypeData());

			if(pMap)
			{
				const float	y = getPositionY() + vec.getY() * fSpeed - 1.5f,
							fx = vec.getX() * fSpeed,
							fz = vec.getZ() * fSpeed;
				float	x1 = getPositionX() + fx,
						z1 = getPositionZ() + fz,
						x2 = x1 + fz,
						z2 = z1 - fx;
				x1 -= fz;
				z1 += fx;

				if(pMap->heightAt(x1, z1) >= y)
				{
					if(pMap->heightAt(x2, z2) >= y)
						return EvadeCause::terrainBoth;
					return EvadeCause::terrainLeft;
				}
				else if(pMap->heightAt(x2, z2) >= y)
					return EvadeCause::terrainRight;
			}

			it++;
		}
		else if((*it)->getCathegoryID() == c_projectile)
			it = objects.erase(it);
		else
			it++;

	return EvadeCause::none;
}

bool VehicleAI::haveToGoHome()
{	return Vector(getPosition() - m_vec_checkpoint).magnitude() > m_fl_checkpointRadius;	}


void VehicleAI::beginToPatrol()
{
	m_state = patrol;
	fEnginePower = 0.1f;
	fire(false);
}

void VehicleAI::beginToAttack(Vehicle *_pTarget)
{
	m_state = attack;
	pTarget = _pTarget;
}

#define random(min, max)\
	(((float)rand() / RAND_MAX) * ((max) - (min)) + min)

void VehicleAI::beginToEvade(const EvadeCause ec)
{
	if(m_state != evade)
		m_lastState = m_state;
	m_state = evade;

	m_fl_stateTimer = 2.0f;
	if(fEnginePower < 0.1)
		fEnginePower = 0.1f;
	fire(false);

	float	fMinF, fMaxF,
			fMinS, fMaxS,
			fMinU, fMaxU,
			rF,	rS,	rU;

	switch(ec)
	{
	case vehicle:
		fMinF = -1.0f;	fMaxF = 1.0f;
		fMinS = -2.0f;	fMaxS = 2.0f;
		fMinU = -2.0f;	fMaxU = 2.0f;
		break;

	case vehicleCentre:
		fMinF = -0.0f;	fMaxF = 0.0f;
		fMinS = -1.0f;	fMaxS = 1.0f;
		fMinU = -1.0f;	fMaxU = 1.0f;
		break;

	case terrainLeft:
		fMinF = -1.0f;	fMaxF = 1.0f;
		fMinS = 0.0f;	fMaxS = 2.0f;
		fMinU = 0.0f;	fMaxU = 2.0f;
		break;

	case terrainRight:
		fMinF = -1.0f;	fMaxF = 1.0f;
		fMinS = -2.0f;	fMaxS = 0.0f;
		fMinU = 0.0f;	fMaxU = 2.0f;
		break;

	case terrainBoth:
		fMinF = -1.0f;	fMaxF = 1.0f;
		fMinS = -1.0f;	fMaxS = 1.0f;
		fMinU = 0.0f;	fMaxU = 2.0f;
		break;
	};

	do
	{
		rF = random(fMinF, fMaxF);
		rS = random(fMinS, fMaxS);
		rU = random(fMinU, fMaxU);
	} while(!rF && !rS && !rU);

	m_vec_destinationDirection = rF * getForward() + rS * getStrafe() + rU * getUp();
	m_vec_destinationDirection.normalise();
}

void VehicleAI::beginToGoHome()
{
	m_state = gohome;
	fEnginePower = 1.0f;
}


void VehicleAI::getAnglesToDestination(float &phi, float &theta)
{
	phi = asin(m_vec_destinationDirection.getY());
	if(!m_vec_destinationDirection.getZ())
		theta = (GLfloat)PI / (m_vec_destinationDirection.getX() > 0.0f ? -2.0f : 2.0f);
	else
	{
		theta = atan(m_vec_destinationDirection.getX() / m_vec_destinationDirection.getZ());
		if(m_vec_destinationDirection.getZ() > 0.0f)
			theta += (GLfloat)PI;
	}

	theta -= fTheta;
	phi -= fPhi;
	
	phi = phi - (float)PIx2 * floor(phi / (float)PIx2 + 0.5f);
	theta = theta - (float)PIx2 * floor(theta / (float)PIx2 + 0.5f);
}

void VehicleAI::steerToDestination(const float time)
{
	float	phi,
			theta;

	getAnglesToDestination(phi, theta);
	changeDirection(theta * time, phi * time);
}


bool VehicleAI::advance(float time)
{
	if(!Vehicle::advance(time))
		return false;
	if(!pScene)
		return false;

	fire(false);

	objects.clear();
	pScene->getObjectsInRadius(objects, this, m_fl_fireRange);
	EvadeCause	ec = haveToEvade(m_state == evade ? m_vec_destinationDirection : getForward());

	if(ec != none)
		beginToEvade(ec);
	else if(haveToGoHome())
		beginToGoHome();

	switch(m_state)
	{
		case patrol:
			advance_patrol(time);
			break;
		case gohome:
			advance_goHome(time);
			break;
		case attack:
			advance_attack(time);
			break;
		case evade:
			advance_evade(time);
			break;
	}

	return true;
}