#include <aquawars/Weapon.h>


Sound	Weapon::sndEmpty("res/sounds/weapon_Empty.wav"),
		Weapon::sndLightMG("res/sounds/weapon_LightMG.wav"),
		Weapon::sndHeavyMG("res/sounds/weapon_HeavyMG.wav"),
		Weapon::sndLightCannon("res/sounds/weapon_LightCannon.wav", false, true),
		Weapon::sndHeavyCannon("res/sounds/weapon_HeavyCannon.wav", false, true),
		Weapon::sndGaussCannon("res/sounds/weapon_GaussCannon.wav", false, true),
		Weapon::sndTorpedoLauncher("res/sounds/weapon_TorpedoLauncher.wav");
const char *	Weapon::strNames[COUNT] = {
		"Leichtes MG",
		"Schweres MG",
		"Leichte Kanone",
		"Schwere Kanone",
		"Gausskanone",
		"Torpedos"
};


////////////////////////////////// WEAPON \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser Weapon-Konstruktor.
/////
////////////////////////////////// WEAPON \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
Weapon::Weapon(Vehicle *_parent, const Type &_type) :
	parent(_parent),
	type(_type),
	fCooldownTimer(0.0f)
{	
	switch(type)
	{
	case LightMG:
		nMaxAmmunition = 500;
		fCooldown = 0.1f;
		fProjectileLife = 3.0f;
		fProjectileSpeed = 75.0f;
		fProjectileDamage = 12.0f;
		break;

	case HeavyMG:
		nMaxAmmunition = 350;
		fCooldown = 0.15f;
		fProjectileLife = 3.0f;
		fProjectileSpeed = 75.0f;
		fProjectileDamage = 24.0f;
		break;

	case LightCannon:
		nMaxAmmunition = 100;
		fCooldown = 1.0f;
		fProjectileLife = 3.0f;
		fProjectileSpeed = 75.0f;
		fProjectileDamage = 50.0f;
		break;

	case HeavyCannon:
		nMaxAmmunition = 75;
		fCooldown = 1.5f;
		fProjectileLife = 3.0f;
		fProjectileSpeed = 75.0f;
		fProjectileDamage = 100.0f;
		break;

	case GaussCannon:
		nMaxAmmunition = 20;
		fCooldown = 3.0f;
		fProjectileLife = 2.0f;
		fProjectileSpeed = 133.3f;
		fProjectileDamage = 300.0f;
		break;

	case TorpedoLauncher:
		nMaxAmmunition = 5;
		fCooldown = 5.0f;
		fProjectileLife = 15.0f;
		fProjectileSpeed = 30.0f;
		fProjectileDamage = 750.0f;
		break;

	default:
		throw NotSupportedError("unsupported weapon type");
	}

	nAmmunition = nMaxAmmunition;
}


Projectile * Weapon::fire(const Vector3<float> &position, const Vector3<float> &forward, const float &speed)
{
	Projectile	*p = NULL;
	Sound		*sound = NULL;
	
	if(ready())
	{
		switch(type)
		{
		case Type::LightMG:
			p = new Bullet(parent, fProjectileLife, fProjectileDamage);
			sound = &sndLightMG;
			break;

		case Type::HeavyMG:
			p = new Bullet(parent, fProjectileLife, fProjectileDamage);
			sound = &sndHeavyMG;
			break;

		case Type::LightCannon:
			p = new Bullet(parent, fProjectileLife, fProjectileDamage);
			sound = &sndLightCannon;
			break;

		case Type::HeavyCannon:
			p = new Bullet(parent, fProjectileLife, fProjectileDamage);
			sound = &sndHeavyCannon;
			break;

		case Type::GaussCannon:
			p = new GaussBullet(parent, fProjectileLife, fProjectileDamage);
			sound = &sndGaussCannon;
			break;

		case Type::TorpedoLauncher:
			p = new Torpedo(parent, fProjectileLife, fProjectileDamage);
			sound = &sndTorpedoLauncher;
			break;

		default:
			throw NotSupportedError("unsupported weapon type");
		}

		fCooldownTimer = fCooldown;
		nAmmunition--;

		float	phi = asin(forward.getY()),
				theta = atan(forward.getX() / forward.getZ());
		if(forward.getZ() > 0.0f)
			theta = (GLfloat)PI + theta;

		p->reset();
		p->postRotateX(phi);
		p->postRotateY(theta);
		p->postTranslate(position);
		p->updateFinal();

		p->speed(fProjectileSpeed + speed);
	}
	else if(!nAmmunition)
		sound = &sndEmpty;

	if(sound)
	{
		sound->set_position(position);
		sound->start();
	}

	return p;
}
