#include <framework/Module.h>
#include <framework/Exception.h>

#define	PREFIX	"__module_"


FRAMEWORK_BEGIN

using namespace std;


const string	Module::strPrefix = string(PREFIX);


Module::Module(const string &strModule, uint_t nMode)
{
#ifdef _WINDOWS
	handle = LoadLibrary(strModule.c_str());
	if(!handle)
		throw InvalidModuleException(strModule + ": " + getLastErrorString());

	typedef	const Version *(*FnGetVersion)();
	FnGetVersion fnGetVersion;

	// try to get version info
	try
	{	fnGetVersion = (FnGetVersion)getSymbol(PREFIX "version_get");	}
	catch(UndefinedSymbolException &use)
	{	throw InvalidModuleException(use);	}

	if(!(pVersion = fnGetVersion()))
		throw InvalidModuleException("invalid version info");
#else
	handle = dlopen(strModule.c_str(), nMode);
	if(!handle)
		throw InvalidModuleException(strModule + ": " + dlerror());

	// try to get version info
	try
	{	pVersion = (const Version *)getSymbol(PREFIX "version");	}
	catch(UndefinedSymbolException &use)
	{	throw InvalidModuleException("version info not found");	}
#endif
}


FRAMEWORK_END
