#include <graphics/Image.h>


GRAPHICS_BEGIN


void Image::_load()
{
	if(!rTexture.isEmpty())
		rTexture->load();
}
void Image::_remove()
{
	if(!rTexture.isEmpty())
		rTexture->remove();
}

void Image::_glLoad()
{
	if(!rTexture.isEmpty())
		rTexture->glLoad();
}
void Image::_glRemove()
{
	if(!rTexture.isEmpty())
		rTexture->glRemove();
}


void Image::_glRender()
{
	glPushAttrib(GL_ENABLE_BIT);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glBegin(GL_QUADS);
		if(fLife >= fFadeOut)
		{
			if(fAge >= fFadeIn)
				glColor4f(fR, fG, fB, fA);
			else
				glColor4f(fR, fG, fB, fA * fAge / fFadeIn);
		}
		else 
			glColor4f(fR, fG, fB, fA * fLife / fFadeOut);

		glTexCoord2f(1.0f, 1.0f);	glVertex2f(fX + fSX, fY);
		glTexCoord2f(1.0f, 0.0f);	glVertex2f(fX + fSX, fY + fSY);
		glTexCoord2f(0.0f, 0.0f);	glVertex2f(fX, fY + fSY);
		glTexCoord2f(0.0f, 1.0f);	glVertex2f(fX, fY);
	glEnd();

	glPopAttrib();
}


Image::Image(	Texture::Reference &_texture, const float &_x, const float &_y, const float &_sx, const float &_sy,
				const float &_r, const float &_g, const float &_b, const float &_a									) :
	rTexture(_texture),
	fX(_x),
	fY(_y),
	fSX(_sx),
	fSY(_sy),
	fR(_r),
	fG(_g),
	fB(_b),
	fA(_a),
	bAging(false)
{	/* nothing to do */	}

Image::Image(	Texture::Reference &_texture, const float &_x, const float &_y, const float &_sx, const float &_sy,
				const float &_r, const float &_g, const float &_b, const float &_a,
				const float &_life, const float &_fadeIn, const float &_fadeOut										) :
	rTexture(_texture),
	fX(_x),
	fY(_y),
	fSX(_sx),
	fSY(_sy),
	fR(_r),
	fG(_g),
	fB(_b),
	fA(_a),
	bAging(true),
	fLife(_life),
	fAge(0.0f),
	fFadeIn(_fadeIn),
	fFadeOut(_fadeOut)
{	/* nothing to do */	}

Image::~Image()
{	_remove();	}


bool Image::advance(float time)
{
	if(!bAging)
		return true;

	fLife -= time;
	fAge += time;
	return fLife > 0.0f;
}


void Image::glReload()
{
	Object2::glReload();

	if(!rTexture.isEmpty())
		rTexture->glReload();
}


void Image::glBind()
{
	if(!rTexture.isEmpty())
		rTexture->glBindLoad();
}


GRAPHICS_END
