#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <framework/Exception.h>

#include <graphics/Model.h> 
//#include "tga.h" 


GRAPHICS_BEGIN

Bone::Bone()	
{
}

Bone::~Bone()	
{
	if(PositionKeyFrames != NULL)
	{
		delete[] PositionKeyFrames;
		PositionKeyFrames = NULL;
	}

	if(RotationKeyFrames != NULL)
	{
		delete[] RotationKeyFrames;
		RotationKeyFrames = NULL;
	}
}

void Bone::render( void )
{
	Vertex	vector;
	Vertex	parentvector;

	vector.set(0,0,0);
	vector = m_final * vector;

	if( Parent != NULL )
	{
		parentvector.set(0,0,0);
		parentvector= Parent->m_final * parentvector;
	}

	glDisable( GL_TEXTURE_2D );

	// render bone as a line
	glLineWidth(1.0);
	glColor3f(1.0, 0, 0);
	glBegin(GL_LINES);
	glVertex3f( vector.getX(), vector.getY(), vector.getZ() );
	if(Parent != NULL)
		glVertex3f( parentvector.getX(), parentvector.getY(), parentvector.getZ() );	
	else
		glVertex3f( vector.getX(), vector.getY(), vector.getZ() );
	glEnd();

	// render bone-ends as fat points
	glPointSize(2.0);
	glColor3f(1.0, 0, 1.0);
	glBegin(GL_POINTS);
	glVertex3f( vector.getX(), vector.getY(), vector.getZ() );
	if(Parent != NULL)
		glVertex3f( parentvector.getX(), parentvector.getY(), parentvector.getZ() );	
	glEnd();
	glColor3f(1.0, 1.0, 1.0);

}

bool Bone::loadFromMs3dAsciiSegment( FILE *file )
{
    char szLine[256];
	int	 j;
	int	 nFlags;

    // name
    if (!fgets (szLine, 256, file))
		return false;

    if (sscanf (szLine, "\"%[^\"]\"", Name) != 1)
		return false;

    // parent name
    if (!fgets (szLine, 256, file))
		return false;
    strcpy (ParentName, "");
    sscanf (szLine, "\"%[^\"]\"", ParentName);


	if (!fgets (szLine, 256, file))
		return false;
    if (sscanf (szLine, "%d %f %f %f %f %f %f",
                    &nFlags,
                    &startPosition[0], &startPosition[1], &startPosition[2],
                    &startRotation[0], &startRotation[1], &startRotation[2]) != 7)
		return false;
 
 
    // position key count
    if (!fgets (szLine, 256, file))
		return false;
    if (sscanf (szLine, "%d", &NumPositionKeys) != 1)
		return false;

	PositionKeyFrames = new KeyFrame[NumPositionKeys];
    for (j = 0; j < NumPositionKeys; j++)
    {
		if (!fgets (szLine, 256, file))
			return false;
        if (sscanf (szLine, "%f %f %f %f", &PositionKeyFrames[j].Time,
			&PositionKeyFrames[j].Value[0],
			&PositionKeyFrames[j].Value[1],
			&PositionKeyFrames[j].Value[2] ) != 4)
 			return false;
    }

    // rotation key count
    if (!fgets (szLine, 256, file))
		return false;
    if (sscanf (szLine, "%d", &NumRotationKeys) != 1)
		return false;

	RotationKeyFrames = new KeyFrame[NumRotationKeys];
    for (j = 0; j < NumRotationKeys; j++)
    {
		if (!fgets (szLine, 256, file))
			return false;
        if (sscanf (szLine, "%f %f %f %f", &RotationKeyFrames[j].Time,
			&RotationKeyFrames[j].Value[0],
			&RotationKeyFrames[j].Value[1],
			&RotationKeyFrames[j].Value[2] ) != 4)
  			return false;
    }
	
	return true;
}

void Bone::initialize( void )
{
	Matrix m_rel;

	// Create a transformation matrix from the position and rotation 
	m_rel = Matrix::getRotationX(startRotation[0]) * Matrix::getRotationY(startRotation[1]) * Matrix::getRotationZ(startRotation[2]);
	m_rel.postTranslate(startPosition[0], startPosition[1], startPosition[2]);

	// Each bone's final matrix is its relative matrix concatenated onto its 
	// parent's final matrix (which in turn is ....)
	//
	if ( Parent == NULL )
		m_final = m_rel;
	else
		m_final = m_rel * Parent->m_final;
}

void Bone::advanceTo( float CurrentTime )
{
	int		i;
	float	deltaTime;
	float	fraction;
	float	Position[3];
	float	Rotation[3];


	/* 
	 * Position
	 */

	// Find appropriate position key frame
	i = 0;
	while( i < NumPositionKeys-1 && PositionKeyFrames[i].Time < CurrentTime )
		i++;

	assert(i < NumPositionKeys);

	if( i > 0 && i < NumPositionKeys-1 )
	{
		// Interpolate between 2 key frames

		// time between the 2 key frames
		deltaTime = PositionKeyFrames[i].Time - PositionKeyFrames[i-1].Time;

		assert( deltaTime > 0 );

		// relative position of interpolation point to the keyframes [0..1]
		fraction = (CurrentTime - PositionKeyFrames[i-1].Time) / deltaTime;

		assert( fraction > 0 );
		assert( fraction < 1.0 );

		Position[0] = PositionKeyFrames[i-1].Value[0] + fraction * (PositionKeyFrames[i].Value[0] - PositionKeyFrames[i-1].Value[0]);
		Position[1] = PositionKeyFrames[i-1].Value[1] + fraction * (PositionKeyFrames[i].Value[1] - PositionKeyFrames[i-1].Value[1]);
		Position[2] = PositionKeyFrames[i-1].Value[2] + fraction * (PositionKeyFrames[i].Value[2] - PositionKeyFrames[i-1].Value[2]);
	}
	else
	{
		memcpy(Position, PositionKeyFrames[i].Value, 3*sizeof(float) );
	}

	/* 
	 * Rotation
	 */

	// Find appropriate rotation key frame
	i = 0;
	while( i < NumRotationKeys-1 && RotationKeyFrames[i].Time < CurrentTime )
		i++;

	assert(i < NumRotationKeys);

	if( i > 0 && i < NumRotationKeys-1 )
	{
		// Interpolate between 2 key frames

		// time between the 2 key frames
		deltaTime = RotationKeyFrames[i].Time - RotationKeyFrames[i-1].Time;
		assert( deltaTime > 0 );

		// relative position of interpolation point to the keyframes [0..1]
		fraction = (CurrentTime - RotationKeyFrames[i-1].Time) / deltaTime;
		assert( fraction > 0 );
		assert( fraction < 1.0 );

		Rotation[0] = RotationKeyFrames[i-1].Value[0] + fraction * (RotationKeyFrames[i].Value[0] - RotationKeyFrames[i-1].Value[0]);
		Rotation[1] = RotationKeyFrames[i-1].Value[1] + fraction * (RotationKeyFrames[i].Value[1] - RotationKeyFrames[i-1].Value[1]);
		Rotation[2] = RotationKeyFrames[i-1].Value[2] + fraction * (RotationKeyFrames[i].Value[2] - RotationKeyFrames[i-1].Value[2]);
	}
	else
	{
		memcpy(Rotation, RotationKeyFrames[i].Value, 3*sizeof(float) );
	}

	// Now we know the position and rotation for this animation frame.
	// Let's calculate the transformation matrix (m_final) for this bone...

	Matrix m_rel;
	Matrix m_frame;

	// Create a transformation matrix from the position and rotation of this
	// joint in the rest position

	m_rel = Matrix::getRotationX(startRotation[0]) * Matrix::getRotationY(startRotation[1]) * Matrix::getRotationZ(startRotation[2]);
	m_rel.postTranslate(startPosition[0], startPosition[1], startPosition[2]);

	// Create a transformation matrix from the position and rotation 
	// m_frame: additional transformation for this frame of the animation 
	m_frame = Matrix::getRotationX(Rotation[0]) * Matrix::getRotationY(Rotation[1]) * Matrix::getRotationZ(Rotation[2]);
	m_frame.postTranslate(Position[0], Position[1], Position[2]);

	// Add the animation state to the rest position
	m_rel = m_rel * m_frame;

	// 
	if ( Parent == NULL )					// this is the root node
		m_final = m_rel;	// m_final := m_rel
	else									// not the root node
	{
		// m_final := parent's m_final * m_rel (matrix concatenation)

		m_final = m_rel * Parent->m_final;
	}
}

Model::Model(const string &_filename) :
	Group3(false, true),
	filename(_filename),
	num_shapes(0),
	shapes(NULL),
	num_materials(0),
	materials(NULL),
	material_indices(NULL),
	bones(NULL)
{	/* nothing to do */	}

Model::~Model()
{	/* _remove(); */	}


void Model::_remove()
{
	for(int i = 0; i < num_shapes; i++)
		shapes[i].remove();

	for(int i = 0; i < num_materials; i++)
		materials[i]->remove();

	if(shapes != NULL)
	{
		delete[] shapes;
		shapes = NULL;
	}
	if(materials != NULL)
	{
		delete[] materials;
		materials = NULL;
	}
	if(material_indices != NULL)
	{
		delete[] material_indices;
		material_indices = NULL;
	}
	if(bones != NULL)
	{
		delete[] bones;
		bones = NULL;
	}
}

void Model::_load()
{
	float	StartTime;
    bool	bError = false;
    char	szLine[256];
    int		nFlags, nIndex, i;

	FILE	*file = fopen (filename.c_str(), "rt");
	if (!file)
		throw IOException("Could not open file");

	CurrentTime = 0;
	num_shapes = -1;

    while (fgets (szLine, 256, file) != NULL  && !bError)
    {
        if (!strncmp (szLine, "//", 2))
            continue;


        int nFrame;
        if (sscanf (szLine, "Frames: %d", &nFrame) == 1)
        {
			MaxTime = 1.0f * nFrame;		// length of animation 
            continue;
        }
        if (sscanf (szLine, "Frame: %d", &nFrame) == 1)
        {
			StartTime = 1.0f * nFrame;
            continue;
        }


        if (sscanf (szLine, "Meshes: %d", &num_shapes) == 1)
        {
			char	szName[MS_MAX_NAME];
	
			shapes = new Shape[num_shapes];
			material_indices = new int[num_shapes];

            for (i = 0; i < num_shapes && !bError; i++)
            {

                if (!fgets (szLine, 256, file))
                {
                    bError = true;
                    break;
                }

                // mesh: name, flags, material index
                if (sscanf (szLine, "\"%[^\"]\" %d %d",szName, &nFlags, &nIndex) != 3)
                {
					bError = true;
                    break;
                }
				material_indices[i] = nIndex;

				if( ! shapes[i].loadFromMs3dAsciiSegment( file ) )
                {
                    bError = true;
                    break;
                }
			}
            continue;
		}

        //
        // materials
        //
       if (sscanf (szLine, "Materials: %d", &num_materials) == 1)
       {
			materials = new Material*[num_materials];

            for (i = 0; i < num_materials && !bError; i++)
            {
				materials[i] = Material::loadFromMs3dAsciiSegment( file );
				if( materials[i] == NULL)
                {
                    bError = true;
                    break;
                }
				materials[i]->load();
			}
            continue;
        }

		
        //
        // bones
        //
        if (sscanf (szLine, "Bones: %d", &num_bones) == 1)
        {
            int i;

			if(num_bones)
				bones = new Bone[num_bones];

            for (i = 0; i < num_bones && !bError; i++)
            {
				if( ! bones[i].loadFromMs3dAsciiSegment(file) )
                {
                    bError = true;
                    break;
                }
            }
            continue;
        }
    }

	fclose (file);

	if(bError || num_shapes <= 0)
		throw IOException("Error while trying to load model!");
		//return false;

	if( !linkBones() )
		throw IOException("Error while trying to load model: Failed to link bones!");
		//return false;


	if(num_shapes)
	{
		float	xmin = shapes[0].vertices[0].getX(), xmax = shapes[0].vertices[0].getX(),
				ymin = shapes[0].vertices[0].getY(), ymax = shapes[0].vertices[0].getY(),
				zmin = shapes[0].vertices[0].getZ(), zmax = shapes[0].vertices[0].getZ(),
				xdiff,
				ydiff,
				zdiff,
				scale;

		for (i = 0; i < num_shapes; i++)
		{
			Shape	*shape = shapes + i;

			for(int j = 0; j < shape->num_vertices; j++)
			{
				const float x = shape->vertices[j].getX(),
							y = shape->vertices[j].getY(),
							z = shape->vertices[j].getZ();

				if(xmin > x)
					xmin = x;
				else if(xmax < x)
					xmax = x;
				if(ymin > y)
					ymin = y;
				else if(ymax < y)
					ymax = y;
				if(zmin > z)
					zmin = z;
				else if(zmax < z)
					zmax = z;
			}
		}

		xdiff = (xmax + xmin) / 2.0f;
		ydiff = (ymax + ymin) / 2.0f;
		zdiff = (zmax + zmin) / 2.0f;

		scale = xmax - xmin;
		if(scale < ymax - ymin)
			scale = ymax - ymin;
		if(scale < zmax - zmin)
			scale = zmax - zmin;

		for (i = 0; i < num_shapes; i++)
		{
			Shape	*shape = shapes + i;

    		if(material_indices[i] >= 0)
				if(materials[material_indices[i]]->hasTexture())
					shapes[i].enableTexture();

			shape->moveAndScale(xdiff, ydiff, zdiff, scale);
			if(material_indices[i] >= 0)
			{
				shape->material(materials[material_indices[i]]);
				if(!materials[material_indices[i]]->hasTexture())
					shape->setCaustics(false);
			}
			else
				shape->setCaustics(false);
			shape->reset();
			shape->parentTransformation(&tmFinal);
			shape->updateFinal();

			add(shapes + i);
		}
	}

	initializeBones();
	attachSkin();
	advance(StartTime);
	//return true;
}

bool Model::linkBones( void )
{
	// The relationship between child and parent bones are defined
	// by each child bone providing the name of its parent bone.
	// This function builds corresponding pointers for faster lookup.
	//
	int i, j;

	// Link children to parent
	//

    for (i = 0; i < num_bones; i++)            
	{
		bones[i].Parent = NULL;

		if( strlen(bones[i].ParentName) >  0 )	// does bone have parent?
		{
			for (j = 0; j < num_bones; j++)	// search for parent
			{
				if( stricmp(bones[j].Name, bones[i].ParentName) == 0)	// j is parent of i
				{
					bones[i].Parent = &bones[j];
					break;
				}
			}
			if ( bones[i].Parent == NULL) // Unable to find parent bone 
				return false;
        }
    }
	
	return true;
}

void Model::initializeBones( void )
{
	int i;

    for (i = 0; i < num_bones; i++)            
	{
		bones[i].initialize();
    }
}

// TODO: Uses private members from various classes (temporarily made public). Clean up!
//
void Model::attachSkin( void )
{
	int		i, j;
	int		bone;
	Matrix	*matrix;
	Vertex	v;


	// Make vertex vector relative to the bone it's attached to

	for( i = 0; i < num_shapes; i++)	// for all shapes
	{	
		for(j = 0; j < shapes[i].num_vertices; j++ )	// for all vertices
		{
			bone = shapes[i].vertices[j].bone;

			if( bone != -1 )	// if vertex attached to a bone
			{
				matrix = & bones[bone].m_final;

				// make relative to bone position and orientation
				// modifies vertex x,y,z 

				v = shapes[i].vertices[j];
				v.inverseTranslateVect(*matrix);
				v.inverseRotateVect(*matrix);
				shapes[i].vertices[j] = v;
			}

		}
	}
}

void Model::renderBones( void )
{
	int	i;

	for(i = 0; i < num_bones; i++)	// for each bone
		bones[i].render();	
}


void Model::reloadTextures( void )
{
	int	i;

	//for(i = 0; i < num_materials; i++)	// for each shape
	//	materials[i].reloadTexture();
}

// renderShadow - renders a horizontal "shadow" at a given height
//					(only works if the shadow is projected onto a flat horizontal surface)
//
void Model::renderShadow( float height )
{
	int	k;
	int	i, j;
	Tri	*tri;
	Vertex	*vec;
	Normal *N;
	Bone *bone;
	Vertex	*v;
	Matrix	*matrix;

	glDisable( GL_TEXTURE_2D );
	//glDisable(GL_LIGHTING);
    glColor3f(0, 0, 0);	       

	for(k = 0; k < num_shapes; k++)	// for each shape
	{
		glBegin(GL_TRIANGLES);
		for(i = 0; i < shapes[k].num_triangles; i++)	// for each triangle
		{
			tri = shapes[k].triangles + i;				// pointer to triangle

			for(j = 0; j < 3; j++)	// 3 vertices of the triangle
			{
			vec = shapes[k].vertices + tri->v[j];		// pointer to vertex
			if( vec->bone == -1 )
			{
				glVertex3f( vec->getX(), height, vec->getZ() );	// project onto shadow plane
			}
			else
			{
				
				bone = &bones[vec->bone];
				matrix = & bone->m_final;
			
				v = vec;
				*v = *matrix * *v;
				// project onto shadow plane
				v->_element(1,0) = height;
				glVertex3f( v->getX(), v->getY(), v->getZ() );	
			}
			}
		}
		glEnd();
	}
}


/*void Model::_glRender()
{
	int	k;
	int	i, j;
	Tri	*tri;
	Vertex	*vec;
	Normal *N;
	Bone *bone;
	Vertex	*v;
	Matrix	*matrix;

    glColor3f(1.0, 1.0, 1.0);	// white       

	for(k = 0; k < num_shapes; k++)	// for each shape
	{
		int materialIndex = material_indices[k];
		if(materialIndex >= 0)
			materials[materialIndex]->glBind();
		shapes[k].glRender();*/



		/*glBegin(GL_TRIANGLES);
		for(i = 0; i < shapes[k].num_triangles; i++)	// for each triangle
		{
			tri = shapes[k].triangles + i;				// pointer to triangle

			for(j = 0; j < 3; j++)	// 3 vertices of the triangle
			{
			N = shapes[k].normals + tri->n[j];
			glNormal3f(N->getX(), N->getY(), N->getZ());	// set normal vector  (object space)

			vec = shapes[k].vertices + tri->v[j];		// pointer to vertex
			glTexCoord2f (vec->u, vec->v);			// texture coordinate

			if( vec->bone == -1 )
			{
				glVertex3f( vec->getX(), vec->getY(), vec->getZ() );	
			}
			else
			{
				bone = &bones[vec->bone];
				matrix = & bone->m_final;
			
				v = vec;
				*v = *matrix * *v;
				glVertex3f( v->getX(), v->getY(), v->getZ() );	
			}
			}
		}
		glEnd();*/
	/*}
}*/


bool Model::advance(float time)
{
	Group3::advance(time);

	CurrentTime += time;
	if( CurrentTime > MaxTime )	// this is a looped animation!
		CurrentTime = 1.0;		// N.B. time starts at 1, not at 0!

    for(int i = 0; i < num_bones; i++)            
		bones[i].advanceTo( CurrentTime );

	return true;
}


void Model::glReload()
{
	Group3::glReload();

	for(uint_t i = 0; i < num_materials; i++)
		materials[i]->glReload();
}


GRAPHICS_END
