#include <graphics/Object3.h>

#include <graphics/Scene.h>

GRAPHICS_BEGIN

#define NUM_PATTERNS 32

GLfloat Object3::fCausticScale = 0.2f;
GLfloat Object3::fCausticTime = 0.0f;
int		Object3::nCausticIncrement = 1;
GLuint	Object3::uiaCausticTextures[NUM_PATTERNS];


void _Object3::boundingBox(const GLfloat &sx, const GLfloat &sy, const GLfloat &sz, const GLfloat &x, const GLfloat &y, const GLfloat &z)
{
	if(pBoundary)
		delete[] pBoundary;

	boundaryType = box;
	pBoundary = new GLfloat[6];

	pBoundary[0] = sx;
	pBoundary[1] = sy;
	pBoundary[2] = sz;
	pBoundary[3] = x;
	pBoundary[4] = y;
	pBoundary[5] = z;
}

void _Object3::boundingSphere(const GLfloat &radius, const GLfloat &x, const GLfloat &y, const GLfloat &z)
{
	if(pBoundary)
		delete[] pBoundary;

	boundaryType = sphere;
	pBoundary = new GLfloat[5];

	pBoundary[0] = radius;
	pBoundary[1] = radius * radius;
	pBoundary[2] = x;
	pBoundary[3] = y;
	pBoundary[4] = z;
}


bool _Object3::_collides_spheres(_Object3 &o1, _Object3 &o2)
{
	const GLfloat	*pb1 = o1.getBoundary(),
					*pb2 = o2.getBoundary();
	const GLfloat	x1 = o1.getPositionX() + pb1[2],
					y1 = o1.getPositionY() + pb1[3],
					z1 = o1.getPositionZ() + pb1[4],
					x2 = o2.getPositionX() + pb2[2],
					y2 = o2.getPositionY() + pb2[3],
					z2 = o2.getPositionZ() + pb2[4],
					x = x1 - x2,
					y = y1 - y2,
					z = z1 - z2;

	return (x * x + y * y + z * z) <= (pb1[1] + pb2[1]);
}

bool _Object3::_collides_sphere_box(_Object3 &o1, _Object3 &o2)
{
	const GLfloat	*pb1 = o1.getBoundary(),
					*pb2 = o2.getBoundary();
	const GLfloat	x1 = o1.getPositionX() + pb1[2],
					y1 = o1.getPositionY() + pb1[3],
					z1 = o1.getPositionZ() + pb1[4],
					x2 = o2.getPositionX() + pb2[3],
					y2 = o2.getPositionY() + pb2[4],
					z2 = o2.getPositionZ() + pb2[5],
					x = x1 - x2,
					y = y1 - y2,
					z = z1 - z2;

	// first do a sphere check (faster!)
	if((x * x + y * y + z * z) > (pb1[1] + pb2[0] * pb2[0] + pb2[1] * pb2[1] + pb2[2] * pb2[2]))
		return false;

	// TODO: find some working code
	return true;
}

bool _Object3::_collides_boxes(_Object3 &o1, _Object3 &o2)
{
	const GLfloat	*pb1 = o1.getBoundary(),
					*pb2 = o2.getBoundary();
	const GLfloat	x1 = o1.getPositionX() + pb1[3],
					y1 = o1.getPositionY() + pb1[4],
					z1 = o1.getPositionZ() + pb1[5],
					x2 = o2.getPositionX() + pb2[3],
					y2 = o2.getPositionY() + pb2[4],
					z2 = o2.getPositionZ() + pb2[5],
					x = x1 - x2,
					y = y1 - y2,
					z = z1 - z2;

	// first do a sphere check (faster!)
	if((x * x + y * y + z * z) > (pb1[0] * pb1[0] + pb1[1] * pb1[1] + pb1[2] * pb1[2] + pb2[0] * pb2[0] + pb2[1] * pb2[1] + pb2[2] * pb2[2]))
		return false;

	// TODO: find some working code
	return true;
}

bool _Object3::_collides(_Object3 &o)
{	return true;	}


_Object3::_Object3(const bool _bCanCollide) :
	Object(),
	bCanCollide(_bCanCollide),
	bVisibleRange(false),
	boundaryType(none),
	pBoundary(NULL),
	nCathegoryID((uint_t)-1),
	nTypeID((uint_t)-1),
	pTypeData(NULL)
{	/* nothing to do */	}

_Object3::~_Object3()
{
	if(pBoundary)
		delete[] pBoundary;
}


void _Object3::getBoundingBox(Vector *box) const
{
	if(!pBoundary)
		return;

	GLfloat	sx,
			sy,
			sz,
			*d;

	if(boundaryType == Boundary::box)
	{
		sx = pBoundary[0] / 2.0f;
		sy = pBoundary[1] / 2.0f;
		sz = pBoundary[2] / 2.0f;
		d = pBoundary + 3;
	}
	else
	{
		sx = sy = sz = sqrt(pBoundary[0]);
		d = pBoundary + 2;
	}

	// generate vectors for the corners
	box[0] = tmGL * _Vector4(d[0] + sx, d[1] + sy, d[2] + sz);
	box[1] = tmGL * _Vector4(d[0] + sx, d[1] + sy, d[2] - sz);
	box[2] = tmGL * _Vector4(d[0] + sx, d[1] - sy, d[2] + sz);
	box[3] = tmGL * _Vector4(d[0] + sx, d[1] - sy, d[2] - sz);
	box[4] = tmGL * _Vector4(d[0] - sx, d[1] + sy, d[2] + sz);
	box[5] = tmGL * _Vector4(d[0] - sx, d[1] + sy, d[2] - sz);
	box[6] = tmGL * _Vector4(d[0] - sx, d[1] - sy, d[2] + sz);
	box[7] = tmGL * _Vector4(d[0] - sx, d[1] - sy, d[2] - sz);
}

GLfloat _Object3::getBoundingRadiusSquared() const
{
	if(!pBoundary)
		return 0.0f;

	if(boundaryType == sphere)
		return pBoundary[1];
	else
		return pBoundary[0] * pBoundary[0] + pBoundary[1] * pBoundary[1] + pBoundary[2] * pBoundary[2];
}

GLfloat _Object3::getBoundingRadius() const
{
	if(!pBoundary)
		return 0.0f;

	if(boundaryType == sphere)
		return pBoundary[1];
	else
		return sqrt(pBoundary[0] * pBoundary[0] + pBoundary[1] * pBoundary[1] + pBoundary[2] * pBoundary[2]);
}


void _Object3::cameraPosition(const Vector &camera)
{	/* nothing to do */	}


bool _Object3::collides(_Object3 &o)
{
	if(!bCanCollide || !o.bCanCollide || boundaryType == none || o.boundaryType == none)
		return false;

	bool bCollision;

	if(boundaryType == sphere)
	{
		if(o.boundaryType == sphere)
			bCollision = _collides_spheres(*this, o);
		else if(boundaryType == box)
			bCollision = _collides_sphere_box(*this, o);
		else
			throw NotImplementedException("unknown bounding object type: " + boundaryType);
	}
	else if(boundaryType == box)
	{
		if(o.boundaryType == sphere)
			bCollision = _collides_sphere_box(o, *this);
		else if(boundaryType == box)
			bCollision = _collides_boxes(*this, o);
		else
			throw NotImplementedException("unknown bounding object type: " + boundaryType);
	}
	else
		throw NotImplementedException("unknown bounding object type: " + boundaryType);

	return bCollision ? _collides(o) : false;
}


void Object3::glGetAll(RenderMap &rm, const Vector &vCameraPosition)
{
	float	x = vCameraPosition.getX() - getGlPositionX(),
			y = vCameraPosition.getY() - getGlPositionY(),
			z = vCameraPosition.getZ() - getGlPositionZ();

	if(boundaryType == sphere)
	{
		x -= pBoundary[2];
		y -= pBoundary[3];
		z -= pBoundary[4];
	}
	else if(boundaryType == box)
	{
		x -= pBoundary[3];
		y -= pBoundary[4];
		z -= pBoundary[5];
	}

	cameraPosition(vCameraPosition);
	rm.insert(RenderMap::value_type(x * x + y * y + z * z, this));
}

void Object3::glGetVisible(RenderMap &rm, Camera *pCamera, const Vector &vCameraPosition)
{
	if(pCamera->visibility(this) != Camera::none)
	{
		float	x = vCameraPosition.getX() - getGlPositionX(),
				y = vCameraPosition.getY() - getGlPositionY(),
				z = vCameraPosition.getZ() - getGlPositionZ();

		if(boundaryType == sphere)
		{
			x -= pBoundary[2];
			y -= pBoundary[3];
			z -= pBoundary[4];
		}
		else if(boundaryType == box)
		{
			x -= pBoundary[3];
			y -= pBoundary[4];
			z -= pBoundary[5];
		}

		cameraPosition(vCameraPosition);
		rm.insert(RenderMap::value_type(x * x + y * y + z * z, this));
	}
}

void Object3::loadCausticTextures()
{
	int width, height;
	int i;
	GLubyte *imageData;
	
	glEnable(GL_TEXTURE_2D);

	glGenTextures(32, uiaCausticTextures);


	// Laden der Kaustik-Texturen und an die Texturnamen in uiaCausticTextures gebunden

	for (i=0; i<NUM_PATTERNS; i += nCausticIncrement)
	{
		char filename[80];

		sprintf(filename, "res/textures/caustics/caust%02d.bw", i);
		fflush(stdout);
		imageData = read_alpha_texture(filename, &width, &height);
		if (imageData == NULL)
		{
			fprintf(stderr, "\n%s: could not load image file\n", filename);
			exit(1);
		}

		glBindTexture(GL_TEXTURE_2D, uiaCausticTextures[i]);
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 1, width, height, GL_LUMINANCE, GL_UNSIGNED_BYTE, imageData);
		
		free(imageData);
	}
}

void Object3::glRender()
{
	glPushMatrix();
	glPushAttrib(GL_ENABLE_BIT);
	glMultTransposeMatrixf(tmGL.getPointer());

	if(pMaterial)
		pMaterial->glBindLoad();
	else
		Material::standard.glBindLoad();
	glBindLoad();

	if(bRenderCaustics)
	{
		glActiveTexture(GL_TEXTURE3);
		glEnable(GL_TEXTURE_2D);
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

		GLfloat sPlane[4] = { 0.05f * fCausticScale, 0.025f * fCausticScale, 0.0f, 0.0f };
		GLfloat tPlane[4] = { 0.0f, 0.025f * fCausticScale, 0.05f * fCausticScale, 0.0f };

		glBindTexture(GL_TEXTURE_2D, uiaCausticTextures[(int)(fCausticTime*32)]);

		// Automatische generierung der Texturkoordinaten
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
		glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlane);
		glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlane);
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);

		_glRender();

		glActiveTexture(GL_TEXTURE3);
		glDisable(GL_TEXTURE_2D);
		glActiveTexture(GL_TEXTURE0);
	}
	else
		_glRender();
		
	glPopMatrix();	
}


GRAPHICS_END
