#include "ParticleList.h"

#include <math.h>


GRAPHICS_BEGIN


Particle	ParticleList::m_par_explosion(false, 15.0f, 1.0f, 0.0f, 0.20f),
			ParticleList::m_par_bubble(true, 1.0f, 0.0f, -0.15f, 0.02f),
			ParticleList::m_par_spark(true, 1.0f, 0.0f, 0.5f, 0.075f, 1.0f, 0.5f, 0.0f),
			ParticleList::m_par_flash(false, 100.0f, 25.0f, 0.0f, 0.45f),
			ParticleList::m_par_cloud(false, 5.0f, 0.2f, 0.0f, 0.1f, 1.0f, 1.0f, 1.0f);


////////////////////////////////// PARTICLE LIST \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser ParticleList-Konstruktor, mit bergabeparametern.
/////
////////////////////////////////// PARTICLE LIST \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
ParticleList::ParticleList()
{	/* nothing to do */	}


void ParticleList::insertBubbleParticle(const Vector &position, const Vector &direction, const float velocity)
{
	float		s = randomSize() * 0.1f;
	Particle	*p = new Particle(m_par_bubble);

	p->deleteOnRemove(true);

	p->size(s);
	p->weight(-s*0.75f);
	
	p->postTranslate(position);
	p->movementDirection(direction * velocity);
	p->updateFinal();

	add(p);
}

void ParticleList::insertBubbleParticle(const Vector &position)
{
	float		s = randomSize() * 0.1f;
	Particle	*p = new Particle(m_par_bubble);
	
	p->deleteOnRemove(true);

	p->size(s);
	p->weight(-s*0.75f);
	
	p->postTranslate(position);
	p->movementDirection(Vector(0.0f, 0.0f, 0.0f));
	p->updateFinal();

	add(p);
}

void ParticleList::insertSparkParticle(const Vector &position, const Vector &direction, const float velocity)
{	
	float		s = randomSize() * 0.1f;
	Particle	*p = new Particle(m_par_spark);
	
	p->deleteOnRemove(true);

	p->size(s);
	p->weight(s*0.75f);
	
	p->postTranslate(position);
	p->movementDirection(direction * velocity);
	p->updateFinal();

	add(p);
}

void ParticleList::insertSparkParticle(const Vector &position)
{	
	float		s = randomSize() * 0.1f;
	Particle	*p = new Particle(m_par_spark);

	p->deleteOnRemove(true);

	p->size(s);
	p->weight(s*0.75f);
	
	p->postTranslate(position);
	p->movementDirection(Vector(0.0f, 0.0f, 0.0f));
	p->updateFinal();

	add(p);
}

void ParticleList::insertCloudParticle(const Vector &position, const Vector &direction, const float velocity)
{
	Particle	*p = new Particle(m_par_cloud);
	float		s = randomSize() * 0.3f;
	
	p->deleteOnRemove(true);

	p->size(s);
	p->weight(s);
	
	p->postTranslate(position);
	p->movementDirection(direction * velocity);
	p->updateFinal();

	add(p);
}

void ParticleList::insertCloudParticle(const Vector &position)
{
	Particle	*p = new Particle(m_par_cloud);
	float		s = randomSize() * 0.3f;
	
	p->deleteOnRemove(true);

	p->size(s);
	p->weight(s);
	
	p->postTranslate(position);
	p->movementDirection(Vector(0.0f, 0.0f, 0.0f));
	p->updateFinal();

	add(p);
}

void ParticleList::insertExplosionParticle(const Vector &position, const float size)
{
	if(size > 0.0f)
	{
		Particle	*p = new Particle(m_par_explosion);

		p->deleteOnRemove(true);

		p->size(size * 0.25f);
		p->postTranslate(position);
		p->updateFinal();

		add(p);
	}
}

void ParticleList::insertFlashParticle(const Vector &position, const float size)
{
	if(size > 0.0f)
	{
		Particle	*p = new Particle(m_par_flash);

		p->deleteOnRemove(true);

		p->size(size * 1.0f);
		p->postTranslate(position);
		p->updateFinal();

		add(p);
	}
}


void ParticleList::insertExplosion3D(const int particleKind, const int number, const Vector &position, const float dispersionVelocity, const Vector &movementDirection)
{
	int h;
	float s, w, v, x, y, z;
	Particle	*pTemplate,
				*p;
	
	// Auswahl des richtigen Partikels.
	switch(particleKind)
	{
	case BUBBLE:
		s = 0.1f;
		w = -7.5f;
		pTemplate = &m_par_bubble;
		break;

	case SPARK:
		s = 0.1f;
		w = 5.0f;
		pTemplate = &m_par_spark;
		break;

	case CLOUD:
		s = 0.3f;
		w = -1.0f;
		pTemplate = &m_par_cloud;
		break;
	}	
	
	// Berechnung der Ausbreitung der Explosionselemente.
	for(h = 0; h != number/3; h++)
	{				
		x = (float)(rand()%100 + 1)/100;
		y = (float)(rand()%100 + 1)/100;
		z = sqrt(1.0f - x*x - y*y);		
		
		x *= dispersionVelocity;
		y *= dispersionVelocity;
		z *= dispersionVelocity;
		
		if(rand()%10 < 6) x = -x ;
		if(rand()%10 < 6) y = -y;
		if(rand()%10 < 6) z = -z;
		
		x += movementDirection.getX();
		y += movementDirection.getY();
		z += movementDirection.getZ();

		v = randomSize()*s;

		p = new Particle(*pTemplate);
		p->deleteOnRemove(true);
		p->size(v);
		p->weight(v*w);
		p->postTranslate(position);
		p->movementDirection(x, y, z);
		p->updateFinal();

		add(p);
	}
	for(; h != (number*2)/3; h++)
	{			
		x = (float)(rand()%100 + 1)/100;
		z = (float)(rand()%100 + 1)/100;
		y = sqrt(1.0f - x*x - z*z);	
		
		x *= dispersionVelocity;
		y *= dispersionVelocity;
		z *= dispersionVelocity;
		
		if(rand()%10 < 6) x = -x;
		if(rand()%10 < 6) y = -y;
		if(rand()%10 < 6) z = -z;
			
		x += movementDirection.getX();
		y += movementDirection.getY();
		z += movementDirection.getZ();

		v = randomSize()*s;
		
		p = new Particle(*pTemplate);
		p->deleteOnRemove(true);
		p->size(v);
		p->weight(v*w);
		p->postTranslate(position);
		p->movementDirection(x, y, z);
		p->updateFinal();

		add(p);
	}
	for(; h != number; h++)
	{	
		y = (float)(rand()%100 + 1)/100;
		z = (float)(rand()%100 + 1)/100;
		x = sqrt(1.0f - y*y - z*z);	

		x *= dispersionVelocity;
		y *= dispersionVelocity;
		z *= dispersionVelocity;
		
		if(rand()%10 < 6) x = -x;
		if(rand()%10 < 6) y = -y;
		if(rand()%10 < 6) z = -z;
		
		x += movementDirection.getX();
		y += movementDirection.getY();
		z += movementDirection.getZ();

		v = randomSize()*s;
		
		p = new Particle(*pTemplate);
		p->deleteOnRemove(true);
		p->size(v);
		p->weight(v*w);
		p->postTranslate(position);
		p->movementDirection(x, y, z);
		p->updateFinal();

		add(p);
	}	
}


/////////////////////////////////// ~PARTICLE LIST\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////	Unser ParticleList-Destruktor.
/////
/////////////////////////////////// ~PARTICLE LIST\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
ParticleList::~ParticleList()
{
}

ParticleList *	ParticleList::createCloudExplosion(const int number, const Vector &position, const float dispersionVelocity, const Vector &movementDirection)
{
	ParticleList *pl = new ParticleList();
	pl->deleteOnRemove(true);

	pl->insertExplosion3D(CLOUD, number, position, 0.25f * dispersionVelocity, movementDirection);

	return pl;
}

ParticleList *	ParticleList::createBubbleExplosion(const int number, const Vector &position, const float dispersionVelocity, const Vector &movementDirection)
{
	ParticleList *pl = new ParticleList();
	pl->deleteOnRemove(true);

	pl->insertExplosion3D(BUBBLE, number, position, 0.25f * dispersionVelocity, movementDirection);

	return pl;
}

ParticleList * ParticleList::createExplosion3D(const float explosionSize, const float flashSize, const float dispersion, const int bubbleNumber, const int sparkNumber, const Vector &position, const Vector &movementDirection)
{
	ParticleList *pl = new ParticleList();
	pl->deleteOnRemove(true);

	if(explosionSize > 0)
		pl->insertExplosionParticle(position, explosionSize);
	if(flashSize > 0)
		pl->insertFlashParticle(position, flashSize);
	if(bubbleNumber > 0)
		pl->insertExplosion3D(BUBBLE, bubbleNumber, position, 0.5f * dispersion, movementDirection);
	if(sparkNumber > 0)
		pl->insertExplosion3D(SPARK, sparkNumber, position, 0.5f * dispersion, movementDirection);

	return pl;
}


GRAPHICS_END
