//*** include files
//*****************
#include <stdexcept>

#include <time.h>

#include <framework/framework.h>
#ifdef _WINDOWS
#include <framework/_windows.h>
#endif

#include <graphics/_gl.h>
#include <graphics/SDLApplication.h>


GRAPHICS_BEGIN


//*** class implementation
//************************
void SDLApplication::_dispatchEvent(SDL_Event &event)
{
	switch(event.type)
	{
	case SDL_ACTIVEEVENT:
		onActivate(event.active);
		break;

	case SDL_KEYDOWN:
		onKeyDown(event.key.keysym);
		break;
	case SDL_KEYUP:
		onKeyUp(event.key.keysym);
		break;

	case SDL_MOUSEMOTION:
		if(bHaveFocus)
		{
			if(!nIgnoreNextMouseMoves)
				onMouseMove(event.motion);
			else
				nIgnoreNextMouseMoves--;
		}
		break;
	case SDL_MOUSEBUTTONDOWN:
		if(bHaveFocus)
			onMouseDown(event.button);
		break;
	case SDL_MOUSEBUTTONUP:
		if(bHaveFocus)
			onMouseUp(event.button);
		break;

	case SDL_JOYAXISMOTION:
		onJoystickAxis(event.jaxis);
		break;
	case SDL_JOYBALLMOTION:
		onJoystickTrackball(event.jball);
		break;
	case SDL_JOYHATMOTION:
		onJoystickHat(event.jhat);
		break;
	case SDL_JOYBUTTONDOWN:
		onJoystickDown(event.jbutton);
		break;
	case SDL_JOYBUTTONUP:
		onJoystickUp(event.jbutton);
		break;

	case SDL_QUIT:
		bContinue = FALSE;
		break;

	case SDL_SYSWMEVENT:
		onWindowManagerEvent(event.syswm);
		break;

	case SDL_VIDEORESIZE:
		onResize(event.resize);
		break;
	case SDL_VIDEOEXPOSE:
		onRedraw(event.expose);
		break;

	case SDL_USEREVENT:
		onUserEvent(event.user);
		break;

	default:
		// handle error
		break;
	}
}


void SDLApplication::onActivate(SDL_ActiveEvent &ae)
{
	if(ae.state != SDL_APPMOUSEFOCUS)
	{
		if(ae.gain)
		{
			SDL_WM_GrabInput(SDL_GRAB_ON);
			SDL_ShowCursor(SDL_DISABLE);

			bHaveFocus = TRUE;
		}
		else
		{
			SDL_WM_GrabInput(SDL_GRAB_OFF);
			SDL_ShowCursor(SDL_ENABLE);

			bHaveFocus = FALSE;
		}
	}

	if(ae.gain)
		nIgnoreNextMouseMoves++;
}

void SDLApplication::onKeyDown(SDL_keysym &keysym)
{	
    switch(keysym.sym)
	{
	case SDLK_F1:
		if(pScene)
			pScene->toggleFlatShaded();
		break;
	case SDLK_F2:
		if(pScene)
			pScene->toggleFramerate();
		break;
	case SDLK_F3:
		if(pScene)
			pScene->toggleWireframe();
		break;
    case SDLK_F4:
		// toggle fullscreen
		toggleFullscreen();
		break;
	case SDLK_F7:
		// toggle caustics
		pScene->toggleCaustics();
		break;
	case SDLK_F8:
		// toggle caustics
		pScene->toggleViewFrustumCulling();
		break;

	case SDLK_ESCAPE:
		exit(0);
        break;
    default:
        break;
    }
}

void SDLApplication::onKeyUp(SDL_keysym &keysym)
{		}

void SDLApplication::onMouseMove(SDL_MouseMotionEvent &mme)
{		}
void SDLApplication::onMouseDown(SDL_MouseButtonEvent &mbe)
{		}
void SDLApplication::onMouseUp(SDL_MouseButtonEvent &mbe)
{		}

void SDLApplication::onJoystickAxis(SDL_JoyAxisEvent &jae)
{		}
void SDLApplication::onJoystickTrackball(SDL_JoyBallEvent &jbe)
{		}
void SDLApplication::onJoystickHat(SDL_JoyHatEvent &jhe)
{		}
void SDLApplication::onJoystickDown(SDL_JoyButtonEvent &jbe)
{		}
void SDLApplication::onJoystickUp(SDL_JoyButtonEvent &jbe)
{		}

void SDLApplication::onWindowManagerEvent(SDL_SysWMEvent &swme)
{		}

void SDLApplication::onResize(SDL_ResizeEvent &re)
{		}
void SDLApplication::onRedraw(SDL_ExposeEvent &ee)
{		}

void SDLApplication::onUserEvent(SDL_UserEvent &ue)
{		}

void SDLApplication::onExit()
{
	if(sdlSurface)
	{
		SDL_FreeSurface(sdlSurface);
		sdlSurface = NULL;
	}
	SDL_Quit();
}


void SDLApplication::_init_glew()
{
	GLenum	glewExitCode = glewInit();
	if(glewExitCode != GLEW_OK)
	{
		const char	*strError = (const char *)glewGetErrorString(glewExitCode);

		if(strError)
			throw GlewException(string(__FUNCTION__) + ": glewInit() failed: " + strError);
		else
			throw GlewException(string(__FUNCTION__) + ": glewInit() failed: unknown error");
	}
}

void SDLApplication::_init(int_t argc, char *argv[])
{
	Application::_init(argc, argv);

	srand((uint_t)time(NULL));

    // initialise SDL - only video required
#ifdef _DEBUG
    if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0)
#else
    if(SDL_Init(SDL_INIT_VIDEO) < 0)
#endif
		throw SDLError(getLastSDLError());

    const SDL_VideoInfo* info = NULL;

    info = SDL_GetVideoInfo();
    if(!info)
		throw SDLError(SDL_GetError());

	// set (default) video mode
	setVideoMode();

	// set window caption
	SDL_WM_SetCaption(strCaption.c_str(), NULL);

	// grab mouse
	SDL_WM_GrabInput(SDL_GRAB_ON);
	SDL_ShowCursor(SDL_DISABLE);
}


void SDLApplication::_setVideoMode()
{
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	sdlSurface = SDL_SetVideoMode(	nWidth, nHeight, 32,
									SDL_OPENGL | (bFullscreen ? SDL_FULLSCREEN : SDL_RESIZABLE)	);
	if(!sdlSurface)
		throw SDLError(SDL_GetError());

	if(!bGlewInitialised)
	{	_init_glew();
		bGlewInitialised = TRUE;	}

	// setup perspective
	glViewport(0, 0, nWidth ? nWidth : 1, nHeight ? nHeight : 1);

	// ignore the next mouse move events - they're caused by switching the video mode
	nIgnoreNextMouseMoves += 2 - bFullscreen;

	// tell the scene to reload
	pScene->glReload();
}

void SDLApplication::_toggleFullscreen()
{
	bFullscreen = !bFullscreen;
	// toggle fullscreen mode
	if(!SDL_WM_ToggleFullScreen(sdlSurface))
	{
		// first method didn't work - try the fallback...
		SDL_FreeSurface(sdlSurface);
		_setVideoMode();
	}
}


SDLApplication::SDLApplication(const string &strCaption) :
	pScene(NULL),
	sdlSurface(NULL),
	bContinue(TRUE),
	bGlewInitialised(FALSE),
	bHaveFocus(TRUE),
	nIgnoreNextMouseMoves(0),
	strCaption(strCaption)
{	/* nothing to do */	}

SDLApplication::~SDLApplication()
{	/* nothing to do */	}


int_t SDLApplication::run()
{
	while(bContinue)
	{
		// handle pending events
		SDL_Event	event;

		while(SDL_PollEvent(&event) && bContinue)
			_dispatchEvent(event);

		// change video mode (if requested)
		if(bUpdateVideoMode)
		{
			if(bToggleFullscreen)
			{
				bFullscreen = !bFullscreen;
				bToggleFullscreen = FALSE;
			}
			_setVideoMode();
			bUpdateVideoMode = FALSE;
		}
		else if(bToggleFullscreen)
		{
			_toggleFullscreen();
			bToggleFullscreen = FALSE;
		}

		// render the scene
		if(pScene)
			pScene->glRender();

		// finally swap buffers
		SDL_GL_SwapBuffers();
	}

	// call exit event handler
	onExit();

	return 0;
}

void SDLApplication::exit(int_t nCode)
{
	SDL_Event	event;
	event.type = SDL_QUIT;

	SDL_PushEvent(&event);

	stop();
	Application::exit(nCode);
}


void SDLApplication::setScene(Scene *_pScene)
{
	mScene.P();
	pScene = _pScene;
	mScene.V();
}

void SDLApplication::setVideoMode(uint_t _nWidth, uint_t _nHeight, framework::BOOL _bFullscreen)
{
	nWidth = _nWidth;
	nHeight = _nHeight;
	bFullscreen = _bFullscreen;
	bUpdateVideoMode = TRUE;
}


GRAPHICS_END


int main(int argc, char *argv[])
{
	if(g_pApp == NULL)
		return 1;

	try
	{
		g_pApp->init(argc, argv);
		return g_pApp->_run();
	}
	catch(framework::Error &e)
	{
		e.report();
		return 2;
	}
	catch(framework::Exception &e)
	{
		e.report();
		return 2;
	}
	catch(exception &e)
	{
#if defined(_WINDOWS)
		MessageBox(NULL, e.what(), "Unhandled Exception", MB_ICONERROR);
#else
		cerr << "unhandled exception: " << e.what() << endl;
#endif
		return 2;
	}
}
