#include <graphics/SDLConditional.h>
#include <graphics/SDLMutex.h>


GRAPHICS_BEGIN


SDLConditional::Allocator	SDLConditional::allocator;


SDL_cond * SDLConditional::Allocator::_new() const
{
	SDL_cond *sdlConditional = SDL_CreateCond();
	if(!sdlConditional)
		throw SDLException(getLastSDLError());
	return sdlConditional;
}

void SDLConditional::Allocator::_delete(SDL_cond *sdlConditional) const
{	SDL_DestroyCond(sdlConditional);	}


SDLConditional::~SDLConditional()
{	/* nothing to do */	}


void SDLConditional::signal()
{
	if(sdlConditional.isEmpty())
		sdlConditional.create();
	if(SDL_CondSignal(*this))
		throw SDLException(getLastSDLError());
}

void SDLConditional::broadcast()
{
	if(sdlConditional.isEmpty())
		sdlConditional.create();
	if(SDL_CondBroadcast(*this))
		throw SDLException(getLastSDLError());
}

void SDLConditional::wait()
{
	if(sdlConditional.isEmpty())
		sdlConditional.create();

	SDLMutex	m;

	m.P();
	if(SDL_CondWait(*this, m))
		throw SDLException(getLastSDLError());
}

void SDLConditional::wait(uint32_t nTimeoutMs)
{
	if(sdlConditional.isEmpty())
		sdlConditional.create();

	SDLMutex	m;
	int_t		nExitCode;

	m.P();
	nExitCode = SDL_CondWaitTimeout(*this, m, nTimeoutMs);
	if(nExitCode)
	{
		if(nExitCode == SDL_MUTEX_TIMEDOUT)
			throw TimeoutException("while waiting for conditional");
		else
			throw SDLException(getLastSDLError());
	}
}


GRAPHICS_END
