#include <stdio.h>
#include <stdlib.h>

#include <assert.h>

#include <NvTriStrip.h>

#include <graphics/Shape.h> 

//*** macros
//**********
#define BUFFER_OFFSET(x)	((uint8_t *)NULL + (x))


GRAPHICS_BEGIN


GLenum Shape::Group::_convertType(uint_t _type)
{
	switch(_type)
	{
	case PT_LIST:
		return GL_TRIANGLES;
	case PT_FAN:
		return GL_TRIANGLE_FAN;
	case PT_STRIP:
		return GL_TRIANGLE_STRIP;

	default:
		throw NotImplementedException();
	};
}

Shape::Shape()
{
	num_vertices = 0;
	file = NULL;
	vertices = NULL;
	num_triangles = 0;
	triangles = NULL;
	num_normals = 0;
	normals = NULL;
	pData = NULL;
	bTexture = false;
}

Shape::~Shape()
{	_remove();	}

void Shape::_remove()
{
	if(vertices != NULL)
	{
		delete[] vertices;
		vertices = NULL;
	}
	if(triangles != NULL)
	{
		delete[] triangles;
		triangles = NULL;
	}
	if(normals != NULL)
	{
		delete[] normals;
		normals = NULL;
	}
	if(pData != NULL)
	{
		delete[] pData;
		pData = NULL;
	}
}

void Shape::_glLoadVertexBufferObject()
{
	// check if an object is loaded
	if(!pData)
		return;

	// generate an OpenGL Buffer Object
	glGenBuffersARB(1, &nID);
	if(nID == 0)
		return;

	// transfer data to buffer
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, nID);
	glBufferDataARB(	GL_ARRAY_BUFFER_ARB,
						(nIndices << 3) * sizeof(GLfloat),
						pData,
						GL_STATIC_DRAW_ARB						);

}

/*void	Shape::_glLoadList()
{

}*/

void Shape::_glLoad()
{
	// VertexBufferObject oder Displaylist erzeugen
	_glLoadVertexBufferObject();
}

void Shape::_glRemove()
{
	// VertexBufferObject lschen;
	if(nID != 0)
	{	glDeleteBuffersARB(1, &nID);
		nID = 0;						}
}

void Shape::_load()
{
}

void Shape::glBind()
{
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, nID);
}


void Shape::_glRenderVertexBufferObject()
{
	// enable client states and select arrays to render
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, GL_FLOAT, 0, BUFFER_OFFSET(0));

	if(pMaterial != NULL && pMaterial->hasTexture())
	{
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, BUFFER_OFFSET(nTextureOffset * sizeof(GLfloat)));
	}
	else
	{
		glPushAttrib(GL_ENABLE_BIT);
		glDisable(GL_TEXTURE_2D);
	}

	glEnableClientState(GL_NORMAL_ARRAY);
	glNormalPointer(GL_FLOAT, 0, BUFFER_OFFSET(nNormalsOffset * sizeof(GLfloat)));
	
	for(list<Group>::iterator it = groups.begin(); it != groups.end(); it++)
		// draw
		glDrawArrays(it->type, it->start, (GLsizei)it->size);

	// disable client states
	glDisableClientState(GL_VERTEX_ARRAY);

	if(pMaterial != NULL && pMaterial->hasTexture())
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	else
		glPopAttrib();

	glDisableClientState(GL_NORMAL_ARRAY);	
}

void	Shape::_glRenderImmediateObject()
{
	GLfloat	*pVertexCoords = pData,
			*pNormals = (pData + nNormalsOffset),
			*pTextureCoords = (pData + nTextureOffset);
	size_t	i;

	if(!bTexture)
	{
		glPushAttrib(GL_ENABLE_BIT);
		glDisable(GL_TEXTURE_2D);
	}
		

	for (list<Group>::iterator it = groups.begin(); it != groups.end(); it++)
	{
		glBegin(it->type);
			if(bTexture)
				for(i = it->start; i < it->start + it->size; i++)
				{
					glNormal3fv(pNormals + i * 3);
					glTexCoord2fv(pTextureCoords + i * 2);
					glVertex3fv(pVertexCoords + i * 3);
				}
			else
				for(i = it->start; i < it->start + it->size; i++)
				{
					glNormal3fv(pNormals + i * 3);
					glVertex3fv(pVertexCoords + i * 3);
				}
		glEnd();
	}

	if(!bTexture)
		glPopAttrib();
}

/*void	Shape::_glRenderList()
{
}*/

void	Shape::_glRender()
{
	//VertexBufferObject rendern
	_glRenderVertexBufferObject();

	//ImmediateObject rendern
	//_glRenderImmediateObject();
}


bool Shape::loadFromFile(string *filename)
{
	FILE	*fp;
	unsigned int	i;

	fp = fopen(filename->c_str(), "r");
	if(fp == NULL)
		return false;
		//throw IOException("Could not load Shape");

	fscanf(fp, "%d\n", &num_vertices );

	vertices = new Vertex[num_vertices];

	for(i = 0 ; i < num_vertices; i++)
	{
		fscanf(fp, "%f %f %f %f %f\n", 
				&vertices[i].x(),
				&vertices[i].y(),
				&vertices[i].z(),
				&vertices[i].u,
				&vertices[i].v );
		vertices[i].u = 1.0f - vertices[i].u;
	}

	fscanf(fp, "%d\n", &num_triangles );

	triangles = new Tri[num_triangles];
	for(i = 0; i < num_triangles; i++)
	{
		fscanf(fp, "%d %d %d\n",
				&triangles[i].v[0],
				&triangles[i].v[1],
				&triangles[i].v[2] );
	}

	fclose(fp);
	
	return true;
}

void Shape::render( void )
{
	unsigned int	i,j;
	Tri	*tri;
	Vertex	*vec;
	Normal *N;

	glBegin(GL_TRIANGLES);
	for(i = 0; i < num_triangles; i++)	// for each triangle
	{
		tri = triangles + i;				// pointer to triangle

		for(j = 0; j < 3; j++)	// 3 vertices of the triangle
		{
			N = normals + tri->n[j];
			glNormal3f(N->getX(), N->getY(), N->getZ());	// set normal vector  (object space)

			vec = vertices + tri->v[j];		// pointer to vertex
			glTexCoord2f (vec->u, vec->v);			// texture coordinate

			glVertex3f( vec->getX(), vec->getY(), vec->getZ() );	// 3d coordinate (object space)
		}
	}
	glEnd();
}

bool Shape::loadFromMs3dAsciiSegment( FILE *file )
{
    bool bError = false;
    char szLine[256];
    int nFlags, nIndex;
	unsigned int i, j;

    //
    // vertices
    //
    if(!fgets(szLine, 256, file))
		return false;

    if(sscanf(szLine, "%d", &num_vertices) != 1)
		return false;
	vertices = new Vertex[num_vertices];

    for(j = 0; j < num_vertices; j++)
    {
        if(!fgets(szLine, 256, file))
			return false;
        if(sscanf(szLine, "%d %f %f %f %f %f %d",
            &nFlags,
            &vertices[j].x(), &vertices[j].y(), &vertices[j].z(),
            &vertices[j].u, &vertices[j].v,
            &vertices[j].bone
            ) != 7)
			return false;
		// adjust the y direction of the texture coordinate
		vertices[j].v = 1.0f - vertices[j].v;
    }

    //
    // normals
    //
    if(!fgets(szLine, 256, file))
		return false;

    if(sscanf(szLine, "%d", &num_normals) != 1)
 		return false;
	normals = new Normal[num_normals];

    for(j = 0; j < num_normals; j++)
    {
        if(!fgets(szLine, 256, file))
			return false;

        if(	sscanf(	szLine, "%f %f %f", 
					&normals[j].x(), &normals[j].y(), &normals[j].z()) != 3	)
	 		return false;
    }

    //
    // triangles
    //
    if(!fgets(szLine, 256, file))
		return false;

    if(sscanf(szLine, "%d", &num_triangles) != 1)
 		return false;
	triangles = new Tri[num_triangles];

	IndexMap indices;
	VertexMap reverse_indices;
	pair<IndexMap::iterator, bool> result;

    for(j = 0; j < num_triangles; j++)
    {
        if (!fgets(szLine, 256, file))
 			return false;
        
        if (sscanf(szLine, "%d %d %d %d %d %d %d %d",
            &nFlags,
            &triangles[j].v[0], &triangles[j].v[1], &triangles[j].v[2],
            &triangles[j].n[0], &triangles[j].n[1], &triangles[j].n[2],
            &nIndex
            ) != 8)
 			return false;
        
		assert(triangles[j].v[0] >= 0);
		assert(triangles[j].v[0] < num_vertices);
		assert(triangles[j].v[1] >= 0);
		assert(triangles[j].v[1] < num_vertices);
		assert(triangles[j].v[2] >= 0);
		assert(triangles[j].v[2] < num_vertices);

		result = indices.insert(IndexMap::value_type(Index(triangles[j].v[0], triangles[j].n[0]), indices.size()));
		if(result.second)
			reverse_indices.insert(VertexMap::value_type(result.first->second, result.first->first));
		result = indices.insert(IndexMap::value_type(Index(triangles[j].v[1], triangles[j].n[1]), indices.size()));
		if(result.second)
			reverse_indices.insert(VertexMap::value_type(result.first->second, result.first->first));
		result = indices.insert(IndexMap::value_type(Index(triangles[j].v[2], triangles[j].n[2]), indices.size()));
		if(result.second)
			reverse_indices.insert(VertexMap::value_type(result.first->second, result.first->first));
	}

	uint16_t *_vertices = new uint16_t[num_triangles * 3];
	IndexMap::const_iterator it;

	for(i = 0; i < num_triangles; i++)
	{
		it = indices.find(Index(triangles[i].v[0], triangles[i].n[0]));
		if(it==indices.end())
			return false; //throw InvalidKeyException();

		_vertices[i * 3] = it->second;

		it = indices.find(Index(triangles[i].v[1], triangles[i].n[1]));
		if(it==indices.end())
			return false; //throw InvalidKeyException();

		_vertices[i * 3 + 1] = it->second;

		it = indices.find(Index(triangles[i].v[2], triangles[i].n[2]));
		if(it==indices.end())
			return false; //throw InvalidKeyException();

		_vertices[i * 3 + 2] = it->second;
	}

	SetCacheSize(CACHESIZE_GEFORCE3);
	SetStitchStrips(false);
	SetMinStripSize(0);

	unsigned short numSections;
	
	//primitive groups from NvTriStrip
	PrimitiveGroup* pPrimitiveGroupsStrip;
	if(!GenerateStrips(_vertices, num_triangles * 3, &pPrimitiveGroupsStrip, &numSections))
		return false; // throw NvTriStripException();
	delete[] _vertices;

	uint_t	pos,
			npos,
			tpos,
			v;

	for(i = 0, nIndices = 0; i < numSections; i++)
		nIndices += pPrimitiveGroupsStrip[i].numIndices;
	pData = new float[nIndices << 3];
	v = 0;
	pos = 0;
	npos = nNormalsOffset = nIndices * 3;
	tpos = nTextureOffset = nIndices * 6;

	VertexMap::const_iterator rit;

	groups.clear();
	for(i = 0; i < numSections; i++)
	{
		groups.push_back(Group(pPrimitiveGroupsStrip[i].type, v, pPrimitiveGroupsStrip[i].numIndices));
		v += pPrimitiveGroupsStrip[i].numIndices;
		for(j = 0; j < pPrimitiveGroupsStrip[i].numIndices; j++)
		{
			rit = reverse_indices.find(pPrimitiveGroupsStrip[i].indices[j]);
			if(rit == reverse_indices.end())
				return false;	// throw InvalidKeyException();
			const Index	&index = rit->second;

			pData[pos++] = vertices[index.v].getX();
			pData[pos++] = vertices[index.v].getY();
			pData[pos++] = vertices[index.v].getZ();

			pData[npos++] = normals[index.n].getX();
			pData[npos++] = normals[index.n].getY();
			pData[npos++] = normals[index.n].getZ();

			pData[tpos++] = vertices[index.v].u;
			pData[tpos++] = vertices[index.v].v;
		}
	}

	delete[] pPrimitiveGroupsStrip;
	return true;
}


GRAPHICS_END
