#include "SimulationClient.h"


GRAPHICS_BEGIN


SimulationClient::SimulationClient() :
	pScene(NULL),
	pServer(NULL),
	bAutoDelete(false),
	bRemovingObjects(false)
{	/* nothing to do */	}

SimulationClient::~SimulationClient()
{
	// should check if all objects have been removed
	// from the scene
}


bool SimulationClient::advance(float time)
{
	if(!bRemovingObjects)
	{
		if(!_advance(time))
		{
			bRemovingObjects = true;
			for(it = objects.begin(); it != objects.end(); it++)
				(*it)->removeFromScene();
			it = objects.begin();
		}

		return true;
	}
	else
	{
		while(it != objects.end())
			if((*it)->getRemovedFromScene())
			{
				if(bAutoDelete)
					(*it)->remove();
				it++;
			}
			else
				return true;

		if(bAutoDelete)
			objects.clear();
		else
		{
			pScene = NULL;
			pServer = NULL;
			bRemovingObjects = false;
		}

		return false;
	}
}


void SimulationClient::load()
{
	for(it = objects.begin(); it != objects.end(); it++)
		(*it)->load();
}

void SimulationClient::remove()
{
	for(it = objects.begin(); it != objects.end(); it++)
		(*it)->remove();
}


void SimulationClient::spawn(SimulationClient *sc)
{
	sc->bAutoDelete = true;
	sc->load();

	sc->scene(pScene);
	pScene->add(*sc);

	pServer->add(sc);
}


GRAPHICS_END
