/****************************************************************/
// include files
/****************************************************************/
#include <framework/RLEInputBuffer.h>

#include <graphics/TGATexture.h>


GRAPHICS_BEGIN


Texture * TGATexture::Allocator::_new() const
{	return NULL;	}
Texture * TGATexture::Allocator::_new(const Id *pId) const
{	return new TGATexture(*pId);	}
Texture * TGATexture::Allocator::_copy(const Texture *pItem) const
{	return new TGATexture(*((TGATexture *)pItem));	}

TGATexture::Allocator * TGATexture::Allocator::__copy() const
{
	Allocator *pNew = new Allocator;
	if(!pNew)
		throw OutOfMemoryError();
	return pNew;
}


/****************************************************************/
// TGATexture implementation
/****************************************************************/
void TGATexture::__load16NoAlpha(InputBuffer *pIB)
{
	// format in file: GGGBBBBB ARRRRRGG, we read, ARRRRRGGGGGBBBBB (A is ignored), we need RGB
	uint_t		n3Width = nWidth * 3,
				n2WidthSkip = nWidthSkip << 1,
				nLineDiff = 6 * nWidth;
	GLfloat		*pCurrent,
				*pRowEnd;
	uint16_t	nTemp;

	pImageData = new GLfloat[n3Width * nHeight];

	try
	{
		for(pCurrent = pImageData + (nHeight - 1) * n3Width; ((int)(pCurrent - pImageData)) >= 0; pCurrent -= nLineDiff)
		{
			// read image data
			for(pRowEnd = pCurrent + n3Width; pCurrent < pRowEnd;)
			{
				(*pIB) >> nTemp;
				(*pCurrent++) = ((nTemp >> 10) & 0x1f) / 31.0f;
				(*pCurrent++) = ((nTemp >> 5) & 0x1f) / 31.0f;
				(*pCurrent++) = ((nTemp) & 0x1f) / 31.0f;
			}

			// skip image data outside the power of two
			pIB->skip(n2WidthSkip);
		}
	}
	catch(exception &)
	{	delete[] pImageData;
		throw;				}
}

void TGATexture::__load16(InputBuffer *pIB)
{
	// format in file: GGGBBBBB ARRRRRGG, we read, ARRRRRGGGGGBBBBB, we need RGBA
	uint_t		n4Width = nWidth << 2,
				n2WidthSkip = nWidthSkip << 1,
				nLineDiff = 6 * nWidth;
	GLfloat		*pCurrent,
				*pRowEnd;
	uint16_t	nTemp;

	pImageData = new GLfloat[n4Width * nHeight];

	try
	{
		for(pCurrent = pImageData + (nHeight - 1) * n4Width; ((int)(pCurrent - pImageData)) >= 0; pCurrent -= nLineDiff)
		{
			// read image data
			for(pRowEnd = pCurrent + n4Width; pCurrent < pRowEnd;)
			{
				(*pIB) >> nTemp;
				(*pCurrent++) = ((nTemp >> 10) & 0x1f) / 31.0f;
				(*pCurrent++) = ((nTemp >> 5) & 0x1f) / 31.0f;
				(*pCurrent++) = ((nTemp) & 0x1f) / 31.0f;
				(*pCurrent++) = 1.0f - (nTemp >> 15);
			}

			// skip image data outside the power of two
			pIB->skip(n2WidthSkip);
		}
	}
	catch(exception &)
	{	delete[] pImageData;
		throw;				}
}

void TGATexture::__load24NoAlpha(InputBuffer *pIB)
{
	// image format in file is BGR, we read BGR, we need RGB
	uint_t		n3Width = nWidth * 3,
				n3WidthSkip = nWidthSkip * 3,
				nLineDiff = 6 * nWidth;
	GLfloat		*pCurrent,
				*pRowEnd;
	uint8_t		nTemp;

	pImageData = new GLfloat[n3Width * nHeight];

	try
	{
		for(pCurrent = pImageData + (nHeight - 1) * n3Width; ((int)(pCurrent - pImageData)) >= 0; pCurrent -= nLineDiff)
		{
			// read image data
			for(pRowEnd = pCurrent + n3Width; pCurrent < pRowEnd; pCurrent += 3)
			{
				(*pIB) >> nTemp;
				pCurrent[2] = nTemp / 255.0f;
				(*pIB) >> nTemp;
				pCurrent[1] = nTemp / 255.0f;
				(*pIB) >> nTemp;
				pCurrent[0] = nTemp / 255.0f;
			}

			// skip image data outside the power of two
			pIB->skip(n3WidthSkip);
		}
	}
	catch(exception &)
	{	delete[] pImageData;
		throw;				}
}

void TGATexture::__load32NoAlpha(InputBuffer *pIB)
{
	// image format in file is BGR, we read BGR, we nee RGB
	uint_t		n4Width = nWidth << 2,
				n4WidthSkip = nWidthSkip << 2,
				nLineDiff = nWidth << 3;
	GLfloat		*pCurrent,
				*pRowEnd;
	uint32_t	nTemp;

	pImageData = new GLfloat[n4Width * nHeight];

	try
	{
		for(pCurrent = pImageData + (nHeight - 1) * n4Width; ((int)(pCurrent - pImageData)) >= 0; pCurrent -= nLineDiff)
		{
			// read image data
			for(pRowEnd = pCurrent + n4Width; pCurrent < pRowEnd;)
			{
				(*pIB) >> nTemp;
				(*pCurrent++) = ((nTemp & 0x00ff0000) >> 16) / 255.0f;
				(*pCurrent++) = ((nTemp & 0x0000ff00) >> 8) / 255.0f;
				(*pCurrent++) = (nTemp & 0x000000ff) / 255.0f;
			}

			// skip image data outside the power of two
			pIB->skip(n4WidthSkip);
		}
	}
	catch(exception &)
	{	delete[] pImageData;
		throw;				}
}

void TGATexture::__load32(InputBuffer *pIB)
{
	// ignore attribute bits (assume 8)
	// image format in file is BGRA, we read ARGB, we need RGBA
	uint_t		n4Width = nWidth << 2,
				n4WidthSkip = nWidthSkip << 2,
				nLineDiff = nWidth << 3;
	GLfloat		*pCurrent,
				*pRowEnd;
	uint32_t	nTemp;

	pImageData = new GLfloat[n4Width * nHeight];

	try
	{
		for(pCurrent = pImageData + (nHeight - 1) * n4Width; ((int)(pCurrent - pImageData)) >= 0; pCurrent -= nLineDiff)
		{
			// read image data
			for(pRowEnd = pCurrent + n4Width; pCurrent < pRowEnd;)
			{
				(*pIB) >> nTemp;
				(*pCurrent++) = ((nTemp >> 16) & 0x000000ff) / 255.0f;
				(*pCurrent++) = ((nTemp >> 8) & 0x000000ff) / 255.0f;
				(*pCurrent++) = (nTemp & 0x000000ff) / 255.0f;
				(*pCurrent++) = 1.0f - (nTemp >> 24) / 255.0f;
			}

			// skip image data outside the power of two
			pIB->skip(n4WidthSkip);
		}
	}
	catch(exception &)
	{	delete[] pImageData;
		throw;				}
}


void TGATexture::_load()
{
	InputBuffer	*pIB;

	if(bRLEd)
	{
		RLEInputBuffer *pRIB = new RLEInputBuffer(strFile, nTexelSize);
		pRIB->skip_uncompressed(nImageDataPosition);
		pIB = pRIB;
	}
	else
	{
		pIB = new InputBuffer(strFile);
		pIB->skip(nImageDataPosition);
	}

	(this->*__load)(pIB);

	delete pIB;
}


TGATexture::TGATexture(const Id &id) :	Texture(	id.clamp,
													id.mipmap	),
										strFile(id.file)
{
	InputBuffer	ib(strFile);

	uint8_t	identificationFieldLength,
			colourMapType,
			imageType;

	ib >> identificationFieldLength;
	ib >> colourMapType;
	if(colourMapType != 0)
		// colour maps are not supported yet
		throw NotSupportedException(getErrorMessage("colour maps are not supported"));
	ib >> imageType;
	if(imageType != 2 && imageType != 10)
		// currently only types 2 and 10 (RLE'd) are supported
		throw NotSupportedException(getErrorMessage("unknown image type: " + imageType));
	if(imageType == 10)
		bRLEd = true;

	// read colour map info
	ib.skip(2);	// map origin
	ib.skip(2);	// map length
	ib.skip(1);	// map entry size

	uint16_t	width,
				height;
	uint8_t		imageDescriptor,
				texelSize;

	// read image specification
	ib.skip(4);	// x origin, y origin
	ib >> width;
	ib >> height;
	if(id.crop)
	{
		nWidth = powerOf2(width);
		nWidthSkip = width - nWidth;
		nHeight = powerOf2(height);
	}
	else
	{
		nWidth = width;
		nWidthSkip = 0;
		nHeight = height;
	}

	ib >> texelSize;
	ib >> imageDescriptor;
	nType = (imageDescriptor & 0x0f) ? GL_RGBA : GL_RGB;

	// set getTexel function for texel type
	_init_type();

	// set load function for image type
	switch(texelSize)
	{
	case 16:
		if(nType == GL_RGBA)
			// load with alpha bit
			__load = &TGATexture::__load16;
		else if(nType == GL_RGB)
			// load without alpha bit
			__load = &TGATexture::__load16NoAlpha;
		break;

	case 24:
		// 24 bit pixels without alpha
		__load = &TGATexture::__load24NoAlpha;
		break;

	case 32:
		// 32 bit pixels
		if(nType == GL_RGBA)
			__load = &TGATexture::__load32;
		else
			__load = &TGATexture::__load32NoAlpha;
		break;

	default:
		// unhandled pixel size
		throw NotSupportedException(getErrorMessage("unsupported texel size: " + nTexelSize));
		break;
	}

	// skip image identification field
	ib.skip(identificationFieldLength);

	// colour map should be read here
	// currently not supported - this code won't be executed if a colour map is present

	// save starting position of image data
	nImageDataPosition = ib.tell();
}


GRAPHICS_END
