#include <framework/_string.h>

#include <graphics/_Freetype.h>


GRAPHICS_BEGIN


void _Freetype::init()
{
#undef __FTERRORS_H__
#define FT_ERROR_START_LIST		if(!mErrors.size())	{
#define FT_ERRORDEF(e, v, s)	mErrors[e] = string(s);
#define FT_ERROR_END_LIST		}

#include FT_ERRORS_H

	FT_Error	code;

	if((code = FT_Init_FreeType(&libFreetype)) != 0)
		throw FreetypeException(getFreetypeErrorMessage("failed to initialise freetype", code));
}

const string & _Freetype::getError(const FT_Error &code)
{
	map<int_t, string>::const_iterator	itError = mErrors.find(code);
	static const string					strUnknown("unknown error");

	if(itError != mErrors.end())
		return itError->second;
	else
		return strUnknown;
}


FT_Face _Freetype::loadFont(const string &name)
{
	FT_Error	code;
	FT_Face		face;

	// load the font
	mLock.P();
	code = FT_New_Face(libFreetype, name.c_str(), 0, &face);
	mLock.V();
	if(code != 0)
		throw FreetypeException(getFreetypeErrorMessage("failed to load font \"" + name + "\"", code));

	return face;
}

void _Freetype::setFontSize(FT_Face &face, const uint_t size)
{
	FT_Error	code;

	mLock.P();
	code = FT_Set_Char_Size(face, size << 6, size << 6, 96, 96);
	mLock.V();

	if(code != 0)
		throw FreetypeException(getFreetypeErrorMessage("failed to set font size to " + size, code));
}


void _Freetype::loadGlyphs(FT_Face &face, FT_BitmapGlyph *glyphs, _Freetype::Character *chars)
{
	FT_Glyph			glyph;
	FT_BitmapGlyph		glyph_bitmap;
	FT_Error			code;

	for(uint_t i = 0; i < 256; i++)
	{
		mLock.P();
		code = FT_Load_Char(face, i, FT_LOAD_DEFAULT);
		mLock.V();
		if(code != 0)
			throw FreetypeException(getFreetypeErrorMessage("FT_Load_Char() failed for " + (char)i, code));
		mLock.P();
		code = FT_Get_Glyph(face->glyph, &glyph);
		mLock.V();
		if(code != 0)
			throw FreetypeException(getFreetypeErrorMessage("FT_Get_Glyph() failed for " + (char)i, code));

		if(glyph->format != FT_GLYPH_FORMAT_BITMAP)
		{
			mLock.P();
			code = FT_Glyph_To_Bitmap(&glyph, FT_RENDER_MODE_NORMAL, 0, 1);
			mLock.V();

			if(code != 0)
			{
				mLock.P();
				FT_Done_Glyph(glyph);
				mLock.V();
				throw FreetypeException(getFreetypeErrorMessage("FT_Glyph_To_Bitmap() failed for " + (char)i, code));
			}
		}

		glyph_bitmap = (FT_BitmapGlyph)glyph;
		glyphs[i] = glyph_bitmap;

		chars[i].nHeight = glyph_bitmap->bitmap.rows;
		chars[i].nWidth = glyph_bitmap->bitmap.width;
		chars[i].nLeft = glyph_bitmap->left;
		chars[i].nTop = glyph_bitmap->top;
		chars[i].nAdvanceX = face->glyph->advance.x >> 6;
	}
}

void _Freetype::freeGlyphs(FT_BitmapGlyph *glyphs)
{
	mLock.P();
	for(uint_t i = 0; i < 256; i++)
		FT_Done_Glyph((FT_Glyph)glyphs[i]);
	mLock.V();
}


GRAPHICS_END
